//////////////////////////////////////////////////////////////////////////
// DC6809DASS.C - Motorola 6809 disassembler
// Author   : Daniel Coulom - danielcoulom@gmail.com
// Web site : http://dcmo5.free.fr
// Created  : 2006-12
// Last updated : 2008-10-24
//
// This file is part of DCMO5 v11.
// 
// DCMO5 v11 is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// DCMO5 v11 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with DCMO5 v11.  If not, see <http://www.gnu.org/licenses/>.
//
//////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>
#include "dcmo5.h"

int n;          //nombre de cycles
int pc;         //adresse desassemblage
char w[10];     //zone de travail
char hexa[20];  //adresse et dump hexa
char param[20]; //parametres

char mnemonique[152][6]={
/* *********** 8-Bit Accumulator and Memory Instructions ********** */
/*00*/ "???",  "ADCA", "ADCB", "ADDA", "ADDB", "ANDA", "ANDB", "ASL",
/*08*/ "ASLA", "ASLB", "ASR",  "ASRA", "ASRB", "BITA", "BITB", "CLR",
/*10*/ "CLRA", "CLRB", "CMPA", "CMPB", "COM",  "COMA", "COMB", "DAA",
/*18*/ "DEC",  "DECA", "DECB", "EORA", "EORB", "INC",  "INCA", "INCB",
/*20*/ "LDA",  "LDB",  "LSL",  "LSLA", "LSLB", "LSR",  "LSRA", "LSRB", 
/*28*/ "MUL",  "NEG",  "NEGA", "NEGB", "ORA",  "ORB",  "ROL",  "ROLA",
/*30*/ "ROLB", "ROR",  "RORA", "RORB", "SBCA", "SBCB", "STA",  "STB",
/*38*/ "SUBA", "SUBB", "TST",  "TSTA", "TSTB", "",     "",     "", 
/* *********** 16-Bit Accumulator and Memory Instructions ********** */
/*40*/ "ADDD", "CMPD", "LDD",  "SEX",  "STD",  "SUBD", "",     "", 
/* *********** Index/Stack Pointer Instructions ******************** */
/*48*/ "CMPS", "CMPU", "CMPX", "CMPY", "EXG",  "LEAS", "LEAU", "LEAX",
/*50*/ "LEAY", "LDS",  "LDU",  "LDX",  "LDY",  "PSHS", "PSHU", "PULS",
/*58*/ "PULU", "STS",  "STU",  "STX",  "STY",  "TFR",  "ABX",  "",  
/* *********** Branch Instructions ********************************* */
/*60*/ "BEQ",  "LBEQ", "BNE",  "LBNE", "BMI",  "LBMI", "BPL",  "LBPL",
/*68*/ "BCS",  "LBCS", "BCC",  "LBCC", "BVS",  "LBVS", "BVC",  "LBVC",
/*70*/ "BGT",  "LBGT", "BGE",  "LBGE", "BLE",  "LBLE", "BLT",  "LBLT",
/*78*/ "BHI",  "LBHI", "BHS",  "LBHS", "BLS",  "LBLS", "BLO",  "LBLO",
/*80*/ "BSR",  "LBSR", "BRA",  "LBRA", "BRN",  "LBRN", "",     "", 
/* *********** Miscellaneous Instructions ************************** */
/*88*/ "ANDCC", "CWAI", "NOP", "ORCC", "JMP",  "JSR",  "RTI",  "RTS",
/*90*/ "SWI",  "SWI2", "SWI3", "SYNC", "",     "",     "",     "",  
};

/*type d'instruction
INV   0x0000 invalide 
INH   0x0100 inherente 
DIR   0x0200 directe
IMM1  0x0300 immediate 1 octet
IMM2  0x0400 immediate 2 octets
REL1  0x0500 relative 1 octet
REL2  0x0600 relative 2 octets
IND   0x0700 indexee
EXT   0x0800 etendue
REG0  0x0a00 EXG/TFR
REG1  0x0b00 PSHS
REG2  0x0c00 PULS
REG3  0x0d00 PSHU
REG4  0x0e00 PULU
*/

//definition des instructions sous forme mmttnn hexadecimal avec :
//- mmOOOO = index mnemonique
//-   tt00 = type d'instruction
//-     nn = nombre de cycles
int instruction[0x300] = {
/*0000: NEG   DIR  06 */ 0x290206,
/*0001: BRN   REL1 03 */ 0x840503,
/*0002: ???   INV  00 */ 0x000000,
/*0003: COM   DIR  06 */ 0x140206,
/*0004: LSR   DIR  06 */ 0x250206,
/*0005: ???   INV  00 */ 0x000000,
/*0006: ROR   DIR  06 */ 0x310206,
/*0007: ASR   DIR  06 */ 0x0a0206,
/*0008: ASL   DIR  06 */ 0x070206,
/*0009: ROL   DIR  06 */ 0x2e0206,
/*000a: DEC   DIR  06 */ 0x180206,
/*000b: ???   INV  00 */ 0x000000,
/*000c: INC   DIR  06 */ 0x1d0206,
/*000d: TST   DIR  06 */ 0x3a0206,
/*000e: JMP   DIR  03 */ 0x8c0203,
/*000f: CLR   DIR  06 */ 0x0f0206,
/*0010: ???   INV  00 */ 0x000000,
/*0011: ???   INV  00 */ 0x000000,
/*0012: NOP   INH  02 */ 0x8a0102,
/*0013: SYNC  INH  04 */ 0x930104,
/*0014: ???   INV  00 */ 0x000000,
/*0015: ???   INV  00 */ 0x000000,
/*0016: LBRA  REL2 05 */ 0x830605,
/*0017: LBSR  REL2 09 */ 0x810609,
/*0018: ???   INV  00 */ 0x000000,
/*0019: DAA   INH  02 */ 0x170102,
/*001a: ORCC  IMM1 03 */ 0x8b0303,
/*001b: ???   INV  00 */ 0x000000,
/*001c: ANDCC IMM1 03 */ 0x880303,
/*001d: SEX   INH  02 */ 0x430102,
/*001e: EXG   REG0 08 */ 0x4c0a08,
/*001f: TFR   REG0 06 */ 0x5d0a06,
/*0020: BRA   REL1 03 */ 0x820503,
/*0021: BRN   REL1 03 */ 0x840503,
/*0022: BHI   REL1 03 */ 0x780503,
/*0023: BLS   REL1 03 */ 0x7c0503,
/*0024: BCC   REL1 03 */ 0x6a0503,
/*0025: BCS   REL1 03 */ 0x680503,
/*0026: BNE   REL1 03 */ 0x620503,
/*0027: BEQ   REL1 03 */ 0x600503,
/*0028: BVC   REL1 03 */ 0x6e0503,
/*0029: BVS   REL1 03 */ 0x6c0503,
/*002a: BPL   REL1 03 */ 0x660503,
/*002b: BMI   REL1 03 */ 0x640503,
/*002c: BGE   REL1 03 */ 0x720503,
/*002d: BLT   REL1 03 */ 0x760503,
/*002e: BGT   REL1 03 */ 0x700503,
/*002f: BLE   REL1 03 */ 0x740503,
/*0030: LEAX  IND  04 */ 0x4f0704,
/*0031: LEAY  IND  04 */ 0x500704,
/*0032: LEAS  IND  04 */ 0x4d0704,
/*0033: LEAU  IND  04 */ 0x4e0704,
/*0034: PSHS  REG1 05 */ 0x550b05,
/*0035: PULS  REG2 05 */ 0x570c05,
/*0036: PSHU  REG3 05 */ 0x560d05,
/*0037: PULU  REG4 05 */ 0x580e05,
/*0038: ???   INV  00 */ 0x000000,
/*0039: RTS   INH  05 */ 0x8f0105,
/*003a: ABX   INH  03 */ 0x5e0103,
/*003b: RTI   INH 164 */ 0x8e01a4,
/*003c: CWAI  INH  20 */ 0x890114,
/*003d: MUL   INH  11 */ 0x28010b,
/*003e: ???   INV  00 */ 0x000000,
/*003f: SWI   IMM1 19 */ 0x900313,
/*0040: NEGA  INH  02 */ 0x2a0102,
/*0041: ???   INV  00 */ 0x000000,
/*0042: ???   INV  00 */ 0x000000,
/*0043: COMA  INH  02 */ 0x150102,
/*0044: LSRA  INH  02 */ 0x260102,
/*0045: ???   INV  00 */ 0x000000,
/*0046: RORA  INH  02 */ 0x320102,
/*0047: ASRA  INH  02 */ 0x0b0102,
/*0048: ASLA  INH  02 */ 0x080102,
/*0049: ROLA  INH  02 */ 0x2f0102,
/*004a: DECA  INH  02 */ 0x190102,
/*004b: ???   INV  00 */ 0x000000,
/*004c: INCA  INH  02 */ 0x1e0102,
/*004d: TSTA  INH  02 */ 0x3b0102,
/*004e: ???   INV  00 */ 0x000000,
/*004f: CLRA  INH  02 */ 0x100102,
/*0050: NEGB  INH  02 */ 0x2b0102,
/*0051: ???   INV  00 */ 0x000000,
/*0052: ???   INV  00 */ 0x000000,
/*0053: COMB  INH  02 */ 0x160102,
/*0054: LSRB  INH  02 */ 0x270102,
/*0055: ???   INV  00 */ 0x000000,
/*0056: RORB  INH  02 */ 0x330102,
/*0057: ASRB  INH  02 */ 0x0c0102,
/*0058: ASLB  INH  02 */ 0x090102,
/*0059: ROLB  INH  02 */ 0x300102,
/*005a: DECB  INH  02 */ 0x1a0102,
/*005b: ???   INV  00 */ 0x000000,
/*005c: INCB  INH  02 */ 0x1f0102,
/*005d: TSTB  INH  02 */ 0x3c0102,
/*005e: ???   INV  00 */ 0x000000,
/*005f: CLRB  INH  02 */ 0x110102,
/*0060: NEG   IND  06 */ 0x290706,
/*0061: ???   INV  00 */ 0x000000,
/*0062: ???   INV  00 */ 0x000000,
/*0063: COM   IND  06 */ 0x140706,
/*0064: LSR   IND  06 */ 0x250706,
/*0065: ???   INV  00 */ 0x000000,
/*0066: ROR   IND  06 */ 0x310706,
/*0067: ASR   IND  06 */ 0x0a0706,
/*0068: ASL   IND  06 */ 0x070706,
/*0069: ROL   IND  06 */ 0x2e0706,
/*006a: DEC   IND  06 */ 0x180706,
/*006b: ???   INV  00 */ 0x000000,
/*006c: INC   IND  06 */ 0x1d0706,
/*006d: TST   IND  06 */ 0x3a0706,
/*006e: JMP   IND  03 */ 0x8c0703,
/*006f: CLR   IND  06 */ 0x0f0706,
/*0070: NEG   EXT  07 */ 0x290807,
/*0071: ???   INV  00 */ 0x000000,
/*0072: ???   INV  00 */ 0x000000,
/*0073: COM   EXT  07 */ 0x140807,
/*0074: LSR   EXT  07 */ 0x250807,
/*0075: ???   INV  00 */ 0x000000,
/*0076: ROR   EXT  07 */ 0x310807,
/*0077: ASR   EXT  07 */ 0x0a0807,
/*0078: ASL   EXT  07 */ 0x070807,
/*0079: ROL   EXT  07 */ 0x2e0807,
/*007a: DEC   EXT  07 */ 0x180807,
/*007b: ???   INV  00 */ 0x000000,
/*007c: INC   EXT  07 */ 0x1d0807,
/*007d: TST   EXT  07 */ 0x3a0807,
/*007e: JMP   EXT  04 */ 0x8c0804,
/*007f: CLR   EXT  07 */ 0x0f0807,
/*0080: SUBA  IMM1 02 */ 0x380302,
/*0081: CMPA  IMM1 02 */ 0x120302,
/*0082: SBCA  IMM1 02 */ 0x340302,
/*0083: SUBD  IMM2 04 */ 0x450404,
/*0084: ANDA  IMM1 02 */ 0x050302,
/*0085: BITA  IMM1 02 */ 0x0d0302,
/*0086: LDA   IMM1 02 */ 0x200302,
/*0087: ???   INV  00 */ 0x000000,
/*0088: EORA  IMM1 02 */ 0x1b0302,
/*0089: ADCA  IMM1 02 */ 0x010302,
/*008a: ORA   IMM1 02 */ 0x2c0302,
/*008b: ADDA  IMM1 02 */ 0x030302,
/*008c: CMPX  IMM2 04 */ 0x4a0404,
/*008d: BSR   REL1 07 */ 0x800507,
/*008e: LDX   IMM2 03 */ 0x530403,
/*008f: ???   INV  00 */ 0x000000,
/*0090: SUBA  DIR  04 */ 0x380204,
/*0091: CMPA  DIR  04 */ 0x120204,
/*0092: SBCA  DIR  04 */ 0x340204,
/*0093: SUBD  DIR  06 */ 0x450206,
/*0094: ANDA  DIR  04 */ 0x050204,
/*0095: BITA  DIR  04 */ 0x0d0204,
/*0096: LDA   DIR  04 */ 0x200204,
/*0097: STA   DIR  04 */ 0x360204,
/*0098: EORA  DIR  04 */ 0x1b0204,
/*0099: ADCA  DIR  04 */ 0x010204,
/*009a: ORA   DIR  04 */ 0x2c0204,
/*009b: ADDA  DIR  04 */ 0x030204,
/*009c: CMPX  DIR  06 */ 0x4a0206,
/*009d: JSR   DIR  07 */ 0x8d0207,
/*009e: LDX   DIR  05 */ 0x530205,
/*009f: STX   DIR  05 */ 0x5b0205,
/*00a0: SUBA  IND  04 */ 0x380704,
/*00a1: CMPA  IND  04 */ 0x120704,
/*00a2: SBCA  IND  04 */ 0x340704,
/*00a3: SUBD  IND  06 */ 0x450706,
/*00a4: ANDA  IND  04 */ 0x050704,
/*00a5: BITA  IND  04 */ 0x0d0704,
/*00a6: LDA   IND  04 */ 0x200704,
/*00a7: STA   IND  04 */ 0x360704,
/*00a8: EORA  IND  04 */ 0x1b0704,
/*00a9: ADCA  IND  04 */ 0x010704,
/*00aa: ORA   IND  04 */ 0x2c0704,
/*00ab: ADDA  IND  04 */ 0x030704,
/*00ac: CMPX  IND  04 */ 0x4a0704,
/*00ad: JSR   IND  05 */ 0x8d0705,
/*00ae: LDX   IND  05 */ 0x530705,
/*00af: STX   IND  05 */ 0x5b0705,
/*00b0: SUBA  EXT  05 */ 0x380805,
/*00b1: CMPA  EXT  05 */ 0x120805,
/*00b2: SBCA  EXT  05 */ 0x340805,
/*00b3: SUBD  EXT  07 */ 0x450807,
/*00b4: ANDA  EXT  05 */ 0x050805,
/*00b5: BITA  EXT  05 */ 0x0d0805,
/*00b6: LDA   EXT  05 */ 0x200805,
/*00b7: STA   EXT  05 */ 0x360805,
/*00b8: EORA  EXT  05 */ 0x1b0805,
/*00b9: ADCA  EXT  05 */ 0x010805,
/*00ba: ORA   EXT  05 */ 0x2c0805,
/*00bb: ADDA  EXT  05 */ 0x030805,
/*00bc: CMPX  EXT  07 */ 0x4a0807,
/*00bd: JSR   EXT  08 */ 0x8d0808,
/*00be: LDX   EXT  06 */ 0x530806,
/*00bf: STX   EXT  06 */ 0x5b0806,
/*00c0: SUBB  IMM1 02 */ 0x390302,
/*00c1: CMPB  IMM1 02 */ 0x130302,
/*00c2: SBCB  IMM1 02 */ 0x350302,
/*00c3: ADDD  IMM2 04 */ 0x400404,
/*00c4: ANDB  IMM1 02 */ 0x060302,
/*00c5: BITB  IMM1 02 */ 0x0e0302,
/*00c6: LDB   IMM1 02 */ 0x210302,
/*00c7: ???   INV  00 */ 0x000000,
/*00c8: EORB  IMM1 02 */ 0x1c0302,
/*00c9: ADCB  IMM1 02 */ 0x020302,
/*00ca: ORB   IMM1 02 */ 0x2d0302,
/*00cb: ADDB  IMM1 02 */ 0x040302,
/*00cc: LDD   IMM2 03 */ 0x420403,
/*00cd: ???   INV  00 */ 0x000000,
/*00ce: LDU   IMM2 03 */ 0x520403,
/*00cf: ???   INV  00 */ 0x000000,
/*00d0: SUBB  DIR  04 */ 0x390204,
/*00d1: CMPB  DIR  04 */ 0x130204,
/*00d2: SBCB  DIR  04 */ 0x350204,
/*00d3: ADDD  DIR  06 */ 0x400206,
/*00d4: ANDB  DIR  04 */ 0x060204,
/*00d5: BITB  DIR  04 */ 0x0e0204,
/*00d6: LDB   DIR  04 */ 0x210204,
/*00d7: STB   DIR  04 */ 0x370204,
/*00d8: EORB  DIR  04 */ 0x1c0204,
/*00d9: ADCB  DIR  04 */ 0x020204,
/*00da: ORB   DIR  04 */ 0x2d0204,
/*00db: ADDB  DIR  04 */ 0x040204,
/*00dc: LDD   DIR  05 */ 0x420205,
/*00dd: STD   DIR  05 */ 0x440205,
/*00de: LDU   DIR  05 */ 0x520205,
/*00df: STU   DIR  05 */ 0x5a0205,
/*00e0: SUBB  IND  04 */ 0x390704,
/*00e1: CMPB  IND  04 */ 0x130704,
/*00e2: SBCB  IND  04 */ 0x350704,
/*00e3: ADDD  IND  06 */ 0x400706,
/*00e4: ANDB  IND  04 */ 0x060704,
/*00e5: BITB  IND  04 */ 0x0e0704,
/*00e6: LDB   IND  04 */ 0x210704,
/*00e7: STB   IND  04 */ 0x370704,
/*00e8: EORB  IND  04 */ 0x1c0704,
/*00e9: ADCB  IND  04 */ 0x020704,
/*00ea: ORB   IND  04 */ 0x2d0704,
/*00eb: ADDB  IND  04 */ 0x040704,
/*00ec: LDD   IND  05 */ 0x420705,
/*00ed: STD   IND  05 */ 0x440705,
/*00ee: LDU   IND  05 */ 0x520705,
/*00ef: STU   IND  05 */ 0x5a0705,
/*00f0: SUBB  EXT  05 */ 0x390805,
/*00f1: CMPB  EXT  05 */ 0x130805,
/*00f2: SBCB  EXT  05 */ 0x350805,
/*00f3: ADDD  EXT  07 */ 0x400807,
/*00f4: ANDB  EXT  05 */ 0x060805,
/*00f5: BITB  EXT  05 */ 0x0e0805,
/*00f6: LDB   EXT  05 */ 0x210805,
/*00f7: STB   EXT  05 */ 0x370805,
/*00f8: EORB  EXT  05 */ 0x1c0805,
/*00f9: ADCB  EXT  05 */ 0x020805,
/*00fa: ORB   EXT  05 */ 0x2d0805,
/*00fb: ADDB  EXT  05 */ 0x040805,
/*00fc: LDD   EXT  06 */ 0x420806,
/*00fd: STD   EXT  06 */ 0x440806,
/*00fe: LDU   EXT  06 */ 0x520806,
/*00ff: STU   EXT  06 */ 0x5a0806,
/*1000: ???   INV  00 */ 0x000000,
/*1001: ???   INV  00 */ 0x000000,
/*1002: ???   INV  00 */ 0x000000,
/*1003: ???   INV  00 */ 0x000000,
/*1004: ???   INV  00 */ 0x000000,
/*1005: ???   INV  00 */ 0x000000,
/*1006: ???   INV  00 */ 0x000000,
/*1007: ???   INV  00 */ 0x000000,
/*1008: ???   INV  00 */ 0x000000,
/*1009: ???   INV  00 */ 0x000000,
/*100a: ???   INV  00 */ 0x000000,
/*100b: ???   INV  00 */ 0x000000,
/*100c: ???   INV  00 */ 0x000000,
/*100d: ???   INV  00 */ 0x000000,
/*100e: ???   INV  00 */ 0x000000,
/*100f: ???   INV  00 */ 0x000000,
/*1010: ???   INV  00 */ 0x000000,
/*1011: ???   INV  00 */ 0x000000,
/*1012: ???   INV  00 */ 0x000000,
/*1013: ???   INV  00 */ 0x000000,
/*1014: ???   INV  00 */ 0x000000,
/*1015: ???   INV  00 */ 0x000000,
/*1016: ???   INV  00 */ 0x000000,
/*1017: ???   INV  00 */ 0x000000,
/*1018: ???   INV  00 */ 0x000000,
/*1019: ???   INV  00 */ 0x000000,
/*101a: ???   INV  00 */ 0x000000,
/*101b: ???   INV  00 */ 0x000000,
/*101c: ???   INV  00 */ 0x000000,
/*101d: ???   INV  00 */ 0x000000,
/*101e: ???   INV  00 */ 0x000000,
/*101f: ???   INV  00 */ 0x000000,
/*1020: ???   INV  00 */ 0x000000,
/*1021: LBRN  REL2 65 */ 0x850641,
/*1022: LBHI  REL2 65 */ 0x790641,
/*1023: LBLS  REL2 65 */ 0x7d0641,
/*1024: LBCC  REL2 65 */ 0x6b0641,
/*1025: LBCS  REL2 65 */ 0x690641,
/*1026: LBNE  REL2 65 */ 0x630641,
/*1027: LBEQ  REL2 65 */ 0x610641,
/*1028: LBVC  REL2 65 */ 0x6f0641,
/*1029: LBVS  REL2 65 */ 0x6d0641,
/*102a: LBPL  REL2 65 */ 0x670641,
/*102b: LBMI  REL2 65 */ 0x650641,
/*102c: LBGE  REL2 65 */ 0x730641,
/*102d: LBLT  REL2 65 */ 0x770641,
/*102e: LBGT  REL2 65 */ 0x710641,
/*102f: LBLE  REL2 65 */ 0x750641,
/*1030: ???   INV  00 */ 0x000000,
/*1031: ???   INV  00 */ 0x000000,
/*1032: ???   INV  00 */ 0x000000,
/*1033: ???   INV  00 */ 0x000000,
/*1034: ???   INV  00 */ 0x000000,
/*1035: ???   INV  00 */ 0x000000,
/*1036: ???   INV  00 */ 0x000000,
/*1037: ???   INV  00 */ 0x000000,
/*1038: ???   INV  00 */ 0x000000,
/*1039: ???   INV  00 */ 0x000000,
/*103a: ???   INV  00 */ 0x000000,
/*103b: ???   INV  00 */ 0x000000,
/*103c: ???   INV  00 */ 0x000000,
/*103d: ???   INV  00 */ 0x000000,
/*103e: ???   INV  00 */ 0x000000,
/*103f: SWI2  INH  20 */ 0x910114,
/*1040: ???   INV  00 */ 0x000000,
/*1041: ???   INV  00 */ 0x000000,
/*1042: ???   INV  00 */ 0x000000,
/*1043: ???   INV  00 */ 0x000000,
/*1044: ???   INV  00 */ 0x000000,
/*1045: ???   INV  00 */ 0x000000,
/*1046: ???   INV  00 */ 0x000000,
/*1047: ???   INV  00 */ 0x000000,
/*1048: ???   INV  00 */ 0x000000,
/*1049: ???   INV  00 */ 0x000000,
/*104a: ???   INV  00 */ 0x000000,
/*104b: ???   INV  00 */ 0x000000,
/*104c: ???   INV  00 */ 0x000000,
/*104d: ???   INV  00 */ 0x000000,
/*104e: ???   INV  00 */ 0x000000,
/*104f: ???   INV  00 */ 0x000000,
/*1050: ???   INV  00 */ 0x000000,
/*1051: ???   INV  00 */ 0x000000,
/*1052: ???   INV  00 */ 0x000000,
/*1053: ???   INV  00 */ 0x000000,
/*1054: ???   INV  00 */ 0x000000,
/*1055: ???   INV  00 */ 0x000000,
/*1056: ???   INV  00 */ 0x000000,
/*1057: ???   INV  00 */ 0x000000,
/*1058: ???   INV  00 */ 0x000000,
/*1059: ???   INV  00 */ 0x000000,
/*105a: ???   INV  00 */ 0x000000,
/*105b: ???   INV  00 */ 0x000000,
/*105c: ???   INV  00 */ 0x000000,
/*105d: ???   INV  00 */ 0x000000,
/*105e: ???   INV  00 */ 0x000000,
/*105f: ???   INV  00 */ 0x000000,
/*1060: ???   INV  00 */ 0x000000,
/*1061: ???   INV  00 */ 0x000000,
/*1062: ???   INV  00 */ 0x000000,
/*1063: ???   INV  00 */ 0x000000,
/*1064: ???   INV  00 */ 0x000000,
/*1065: ???   INV  00 */ 0x000000,
/*1066: ???   INV  00 */ 0x000000,
/*1067: ???   INV  00 */ 0x000000,
/*1068: ???   INV  00 */ 0x000000,
/*1069: ???   INV  00 */ 0x000000,
/*106a: ???   INV  00 */ 0x000000,
/*106b: ???   INV  00 */ 0x000000,
/*106c: ???   INV  00 */ 0x000000,
/*106d: ???   INV  00 */ 0x000000,
/*106e: ???   INV  00 */ 0x000000,
/*106f: ???   INV  00 */ 0x000000,
/*1070: ???   INV  00 */ 0x000000,
/*1071: ???   INV  00 */ 0x000000,
/*1072: ???   INV  00 */ 0x000000,
/*1073: ???   INV  00 */ 0x000000,
/*1074: ???   INV  00 */ 0x000000,
/*1075: ???   INV  00 */ 0x000000,
/*1076: ???   INV  00 */ 0x000000,
/*1077: ???   INV  00 */ 0x000000,
/*1078: ???   INV  00 */ 0x000000,
/*1079: ???   INV  00 */ 0x000000,
/*107a: ???   INV  00 */ 0x000000,
/*107b: ???   INV  00 */ 0x000000,
/*107c: ???   INV  00 */ 0x000000,
/*107d: ???   INV  00 */ 0x000000,
/*107e: ???   INV  00 */ 0x000000,
/*107f: ???   INV  00 */ 0x000000,
/*1080: ???   INV  00 */ 0x000000,
/*1081: ???   INV  00 */ 0x000000,
/*1082: ???   INV  00 */ 0x000000,
/*1083: CMPD  IMM2 05 */ 0x410405,
/*1084: ???   INV  00 */ 0x000000,
/*1085: ???   INV  00 */ 0x000000,
/*1086: ???   INV  00 */ 0x000000,
/*1087: ???   INV  00 */ 0x000000,
/*1088: ???   INV  00 */ 0x000000,
/*1089: ???   INV  00 */ 0x000000,
/*108a: ???   INV  00 */ 0x000000,
/*108b: ???   INV  00 */ 0x000000,
/*108c: CMPY  IMM2 05 */ 0x4b0405,
/*108d: ???   INV  00 */ 0x000000,
/*108e: LDY   IMM2 04 */ 0x540404,
/*108f: ???   INV  00 */ 0x000000,
/*1090: ???   INV  00 */ 0x000000,
/*1091: ???   INV  00 */ 0x000000,
/*1092: ???   INV  00 */ 0x000000,
/*1093: CMPD  DIR  07 */ 0x410207,
/*1094: ???   INV  00 */ 0x000000,
/*1095: ???   INV  00 */ 0x000000,
/*1096: ???   INV  00 */ 0x000000,
/*1097: ???   INV  00 */ 0x000000,
/*1098: ???   INV  00 */ 0x000000,
/*1099: ???   INV  00 */ 0x000000,
/*109a: ???   INV  00 */ 0x000000,
/*109b: ???   INV  00 */ 0x000000,
/*109c: CMPY  DIR  07 */ 0x4b0207,
/*109d: ???   INV  00 */ 0x000000,
/*109e: LDY   DIR  06 */ 0x540206,
/*109f: STY   DIR  06 */ 0x5c0206,
/*10a0: ???   INV  00 */ 0x000000,
/*10a1: ???   INV  00 */ 0x000000,
/*10a2: ???   INV  00 */ 0x000000,
/*10a3: CMPD  IND  07 */ 0x410707,
/*10a4: ???   INV  00 */ 0x000000,
/*10a5: ???   INV  00 */ 0x000000,
/*10a6: ???   INV  00 */ 0x000000,
/*10a7: ???   INV  00 */ 0x000000,
/*10a8: ???   INV  00 */ 0x000000,
/*10a9: ???   INV  00 */ 0x000000,
/*10aa: ???   INV  00 */ 0x000000,
/*10ab: ???   INV  00 */ 0x000000,
/*10ac: CMPY  IND  07 */ 0x4b0707,
/*10ad: ???   INV  00 */ 0x000000,
/*10ae: LDY   IND  06 */ 0x540706,
/*10af: STY   IND  06 */ 0x5c0706,
/*10b0: ???   INV  00 */ 0x000000,
/*10b1: ???   INV  00 */ 0x000000,
/*10b2: ???   INV  00 */ 0x000000,
/*10b3: CMPD  EXT  08 */ 0x410808,
/*10b4: ???   INV  00 */ 0x000000,
/*10b5: ???   INV  00 */ 0x000000,
/*10b6: ???   INV  00 */ 0x000000,
/*10b7: ???   INV  00 */ 0x000000,
/*10b8: ???   INV  00 */ 0x000000,
/*10b9: ???   INV  00 */ 0x000000,
/*10ba: ???   INV  00 */ 0x000000,
/*10bb: ???   INV  00 */ 0x000000,
/*10bc: CMPY  EXT  08 */ 0x4b0808,
/*10bd: ???   INV  00 */ 0x000000,
/*10be: LDY   EXT  07 */ 0x540807,
/*10bf: STY   EXT  07 */ 0x5c0807,
/*10c0: ???   INV  00 */ 0x000000,
/*10c1: ???   INV  00 */ 0x000000,
/*10c2: ???   INV  00 */ 0x000000,
/*10c3: ???   INV  00 */ 0x000000,
/*10c4: ???   INV  00 */ 0x000000,
/*10c5: ???   INV  00 */ 0x000000,
/*10c6: ???   INV  00 */ 0x000000,
/*10c7: ???   INV  00 */ 0x000000,
/*10c8: ???   INV  00 */ 0x000000,
/*10c9: ???   INV  00 */ 0x000000,
/*10ca: ???   INV  00 */ 0x000000,
/*10cb: ???   INV  00 */ 0x000000,
/*10cc: ???   INV  00 */ 0x000000,
/*10cd: ???   INV  00 */ 0x000000,
/*10ce: LDS   IMM2 04 */ 0x510404,
/*10cf: ???   INV  00 */ 0x000000,
/*10d0: ???   INV  00 */ 0x000000,
/*10d1: ???   INV  00 */ 0x000000,
/*10d2: ???   INV  00 */ 0x000000,
/*10d3: ???   INV  00 */ 0x000000,
/*10d4: ???   INV  00 */ 0x000000,
/*10d5: ???   INV  00 */ 0x000000,
/*10d6: ???   INV  00 */ 0x000000,
/*10d7: ???   INV  00 */ 0x000000,
/*10d8: ???   INV  00 */ 0x000000,
/*10d9: ???   INV  00 */ 0x000000,
/*10da: ???   INV  00 */ 0x000000,
/*10db: ???   INV  00 */ 0x000000,
/*10dc: ???   INV  00 */ 0x000000,
/*10dd: ???   INV  00 */ 0x000000,
/*10de: LDS   DIR  06 */ 0x510206,
/*10df: STS   DIR  06 */ 0x590206,
/*10e0: ???   INV  00 */ 0x000000,
/*10e1: ???   INV  00 */ 0x000000,
/*10e2: ???   INV  00 */ 0x000000,
/*10e3: ???   INV  00 */ 0x000000,
/*10e4: ???   INV  00 */ 0x000000,
/*10e5: ???   INV  00 */ 0x000000,
/*10e6: ???   INV  00 */ 0x000000,
/*10e7: ???   INV  00 */ 0x000000,
/*10e8: ???   INV  00 */ 0x000000,
/*10e9: ???   INV  00 */ 0x000000,
/*10ea: ???   INV  00 */ 0x000000,
/*10eb: ???   INV  00 */ 0x000000,
/*10ec: ???   INV  00 */ 0x000000,
/*10ed: ???   INV  00 */ 0x000000,
/*10ee: LDS   IND  06 */ 0x510706,
/*10ef: STS   IND  06 */ 0x590706,
/*10f0: ???   INV  00 */ 0x000000,
/*10f1: ???   INV  00 */ 0x000000,
/*10f2: ???   INV  00 */ 0x000000,
/*10f3: ???   INV  00 */ 0x000000,
/*10f4: ???   INV  00 */ 0x000000,
/*10f5: ???   INV  00 */ 0x000000,
/*10f6: ???   INV  00 */ 0x000000,
/*10f7: ???   INV  00 */ 0x000000,
/*10f8: ???   INV  00 */ 0x000000,
/*10f9: ???   INV  00 */ 0x000000,
/*10fa: ???   INV  00 */ 0x000000,
/*10fb: ???   INV  00 */ 0x000000,
/*10fc: ???   INV  00 */ 0x000000,
/*10fd: ???   INV  00 */ 0x000000,
/*10fe: LDS   EXT  07 */ 0x510807,
/*10ff: STS   EXT  07 */ 0x590807,
/*1100: ???   INV  00 */ 0x000000,
/*1101: ???   INV  00 */ 0x000000,
/*1102: ???   INV  00 */ 0x000000,
/*1103: ???   INV  00 */ 0x000000,
/*1104: ???   INV  00 */ 0x000000,
/*1105: ???   INV  00 */ 0x000000,
/*1106: ???   INV  00 */ 0x000000,
/*1107: ???   INV  00 */ 0x000000,
/*1108: ???   INV  00 */ 0x000000,
/*1109: ???   INV  00 */ 0x000000,
/*110a: ???   INV  00 */ 0x000000,
/*110b: ???   INV  00 */ 0x000000,
/*110c: ???   INV  00 */ 0x000000,
/*110d: ???   INV  00 */ 0x000000,
/*110e: ???   INV  00 */ 0x000000,
/*110f: ???   INV  00 */ 0x000000,
/*1110: ???   INV  00 */ 0x000000,
/*1111: ???   INV  00 */ 0x000000,
/*1112: ???   INV  00 */ 0x000000,
/*1113: ???   INV  00 */ 0x000000,
/*1114: ???   INV  00 */ 0x000000,
/*1115: ???   INV  00 */ 0x000000,
/*1116: ???   INV  00 */ 0x000000,
/*1117: ???   INV  00 */ 0x000000,
/*1118: ???   INV  00 */ 0x000000,
/*1119: ???   INV  00 */ 0x000000,
/*111a: ???   INV  00 */ 0x000000,
/*111b: ???   INV  00 */ 0x000000,
/*111c: ???   INV  00 */ 0x000000,
/*111d: ???   INV  00 */ 0x000000,
/*111e: ???   INV  00 */ 0x000000,
/*111f: ???   INV  00 */ 0x000000,
/*1120: ???   INV  00 */ 0x000000,
/*1121: ???   INV  00 */ 0x000000,
/*1122: ???   INV  00 */ 0x000000,
/*1123: ???   INV  00 */ 0x000000,
/*1124: ???   INV  00 */ 0x000000,
/*1125: ???   INV  00 */ 0x000000,
/*1126: ???   INV  00 */ 0x000000,
/*1127: ???   INV  00 */ 0x000000,
/*1128: ???   INV  00 */ 0x000000,
/*1129: ???   INV  00 */ 0x000000,
/*112a: ???   INV  00 */ 0x000000,
/*112b: ???   INV  00 */ 0x000000,
/*112c: ???   INV  00 */ 0x000000,
/*112d: ???   INV  00 */ 0x000000,
/*112e: ???   INV  00 */ 0x000000,
/*112f: ???   INV  00 */ 0x000000,
/*1130: ???   INV  00 */ 0x000000,
/*1131: ???   INV  00 */ 0x000000,
/*1132: ???   INV  00 */ 0x000000,
/*1133: ???   INV  00 */ 0x000000,
/*1134: ???   INV  00 */ 0x000000,
/*1135: ???   INV  00 */ 0x000000,
/*1136: ???   INV  00 */ 0x000000,
/*1137: ???   INV  00 */ 0x000000,
/*1138: ???   INV  00 */ 0x000000,
/*1139: ???   INV  00 */ 0x000000,
/*113a: ???   INV  00 */ 0x000000,
/*113b: ???   INV  00 */ 0x000000,
/*113c: ???   INV  00 */ 0x000000,
/*113d: ???   INV  00 */ 0x000000,
/*113e: ???   INV  00 */ 0x000000,
/*113f: SWI3  INH  20 */ 0x920114,
/*1140: ???   INV  00 */ 0x000000,
/*1141: ???   INV  00 */ 0x000000,
/*1142: ???   INV  00 */ 0x000000,
/*1143: ???   INV  00 */ 0x000000,
/*1144: ???   INV  00 */ 0x000000,
/*1145: ???   INV  00 */ 0x000000,
/*1146: ???   INV  00 */ 0x000000,
/*1147: ???   INV  00 */ 0x000000,
/*1148: ???   INV  00 */ 0x000000,
/*1149: ???   INV  00 */ 0x000000,
/*114a: ???   INV  00 */ 0x000000,
/*114b: ???   INV  00 */ 0x000000,
/*114c: ???   INV  00 */ 0x000000,
/*114d: ???   INV  00 */ 0x000000,
/*114e: ???   INV  00 */ 0x000000,
/*114f: ???   INV  00 */ 0x000000,
/*1150: ???   INV  00 */ 0x000000,
/*1151: ???   INV  00 */ 0x000000,
/*1152: ???   INV  00 */ 0x000000,
/*1153: ???   INV  00 */ 0x000000,
/*1154: ???   INV  00 */ 0x000000,
/*1155: ???   INV  00 */ 0x000000,
/*1156: ???   INV  00 */ 0x000000,
/*1157: ???   INV  00 */ 0x000000,
/*1158: ???   INV  00 */ 0x000000,
/*1159: ???   INV  00 */ 0x000000,
/*115a: ???   INV  00 */ 0x000000,
/*115b: ???   INV  00 */ 0x000000,
/*115c: ???   INV  00 */ 0x000000,
/*115d: ???   INV  00 */ 0x000000,
/*115e: ???   INV  00 */ 0x000000,
/*115f: ???   INV  00 */ 0x000000,
/*1160: ???   INV  00 */ 0x000000,
/*1161: ???   INV  00 */ 0x000000,
/*1162: ???   INV  00 */ 0x000000,
/*1163: ???   INV  00 */ 0x000000,
/*1164: ???   INV  00 */ 0x000000,
/*1165: ???   INV  00 */ 0x000000,
/*1166: ???   INV  00 */ 0x000000,
/*1167: ???   INV  00 */ 0x000000,
/*1168: ???   INV  00 */ 0x000000,
/*1169: ???   INV  00 */ 0x000000,
/*116a: ???   INV  00 */ 0x000000,
/*116b: ???   INV  00 */ 0x000000,
/*116c: ???   INV  00 */ 0x000000,
/*116d: ???   INV  00 */ 0x000000,
/*116e: ???   INV  00 */ 0x000000,
/*116f: ???   INV  00 */ 0x000000,
/*1170: ???   INV  00 */ 0x000000,
/*1171: ???   INV  00 */ 0x000000,
/*1172: ???   INV  00 */ 0x000000,
/*1173: ???   INV  00 */ 0x000000,
/*1174: ???   INV  00 */ 0x000000,
/*1175: ???   INV  00 */ 0x000000,
/*1176: ???   INV  00 */ 0x000000,
/*1177: ???   INV  00 */ 0x000000,
/*1178: ???   INV  00 */ 0x000000,
/*1179: ???   INV  00 */ 0x000000,
/*117a: ???   INV  00 */ 0x000000,
/*117b: ???   INV  00 */ 0x000000,
/*117c: ???   INV  00 */ 0x000000,
/*117d: ???   INV  00 */ 0x000000,
/*117e: ???   INV  00 */ 0x000000,
/*117f: ???   INV  00 */ 0x000000,
/*1180: ???   INV  00 */ 0x000000,
/*1181: ???   INV  00 */ 0x000000,
/*1182: ???   INV  00 */ 0x000000,
/*1183: CMPU  IMM2 05 */ 0x490405,
/*1184: ???   INV  00 */ 0x000000,
/*1185: ???   INV  00 */ 0x000000,
/*1186: ???   INV  00 */ 0x000000,
/*1187: ???   INV  00 */ 0x000000,
/*1188: ???   INV  00 */ 0x000000,
/*1189: ???   INV  00 */ 0x000000,
/*118a: ???   INV  00 */ 0x000000,
/*118b: ???   INV  00 */ 0x000000,
/*118c: CMPS  IMM2 05 */ 0x480405,
/*118d: ???   INV  00 */ 0x000000,
/*118e: ???   INV  00 */ 0x000000,
/*118f: ???   INV  00 */ 0x000000,
/*1190: ???   INV  00 */ 0x000000,
/*1191: ???   INV  00 */ 0x000000,
/*1192: ???   INV  00 */ 0x000000,
/*1193: CMPU  DIR  07 */ 0x490207,
/*1194: ???   INV  00 */ 0x000000,
/*1195: ???   INV  00 */ 0x000000,
/*1196: ???   INV  00 */ 0x000000,
/*1197: ???   INV  00 */ 0x000000,
/*1198: ???   INV  00 */ 0x000000,
/*1199: ???   INV  00 */ 0x000000,
/*119a: ???   INV  00 */ 0x000000,
/*119b: ???   INV  00 */ 0x000000,
/*119c: CMPS  DIR  07 */ 0x480207,
/*119d: ???   INV  00 */ 0x000000,
/*119e: ???   INV  00 */ 0x000000,
/*119f: ???   INV  00 */ 0x000000,
/*11a0: ???   INV  00 */ 0x000000,
/*11a1: ???   INV  00 */ 0x000000,
/*11a2: ???   INV  00 */ 0x000000,
/*11a3: CMPU  IND  07 */ 0x490707,
/*11a4: ???   INV  00 */ 0x000000,
/*11a5: ???   INV  00 */ 0x000000,
/*11a6: ???   INV  00 */ 0x000000,
/*11a7: ???   INV  00 */ 0x000000,
/*11a8: ???   INV  00 */ 0x000000,
/*11a9: ???   INV  00 */ 0x000000,
/*11aa: ???   INV  00 */ 0x000000,
/*11ab: ???   INV  00 */ 0x000000,
/*11ac: CMPS  IND  07 */ 0x480707,
/*11ad: ???   INV  00 */ 0x000000,
/*11ae: ???   INV  00 */ 0x000000,
/*11af: ???   INV  00 */ 0x000000,
/*11b0: ???   INV  00 */ 0x000000,
/*11b1: ???   INV  00 */ 0x000000,
/*11b2: ???   INV  00 */ 0x000000,
/*11b3: CMPU  EXT  08 */ 0x490808,
/*11b4: ???   INV  00 */ 0x000000,
/*11b5: ???   INV  00 */ 0x000000,
/*11b6: ???   INV  00 */ 0x000000,
/*11b7: ???   INV  00 */ 0x000000,
/*11b8: ???   INV  00 */ 0x000000,
/*11b9: ???   INV  00 */ 0x000000,
/*11ba: ???   INV  00 */ 0x000000,
/*11bb: ???   INV  00 */ 0x000000,
/*11bc: CMPS  EXT  08 */ 0x480808,
/*11bd: ???   INV  00 */ 0x000000,
/*11be: ???   INV  00 */ 0x000000,
/*11bf: ???   INV  00 */ 0x000000,
/*11c0: ???   INV  00 */ 0x000000,
/*11c1: ???   INV  00 */ 0x000000,
/*11c2: ???   INV  00 */ 0x000000,
/*11c3: ???   INV  00 */ 0x000000,
/*11c4: ???   INV  00 */ 0x000000,
/*11c5: ???   INV  00 */ 0x000000,
/*11c6: ???   INV  00 */ 0x000000,
/*11c7: ???   INV  00 */ 0x000000,
/*11c8: ???   INV  00 */ 0x000000,
/*11c9: ???   INV  00 */ 0x000000,
/*11ca: ???   INV  00 */ 0x000000,
/*11cb: ???   INV  00 */ 0x000000,
/*11cc: ???   INV  00 */ 0x000000,
/*11cd: ???   INV  00 */ 0x000000,
/*11ce: ???   INV  00 */ 0x000000,
/*11cf: ???   INV  00 */ 0x000000,
/*11d0: ???   INV  00 */ 0x000000,
/*11d1: ???   INV  00 */ 0x000000,
/*11d2: ???   INV  00 */ 0x000000,
/*11d3: ???   INV  00 */ 0x000000,
/*11d4: ???   INV  00 */ 0x000000,
/*11d5: ???   INV  00 */ 0x000000,
/*11d6: ???   INV  00 */ 0x000000,
/*11d7: ???   INV  00 */ 0x000000,
/*11d8: ???   INV  00 */ 0x000000,
/*11d9: ???   INV  00 */ 0x000000,
/*11da: ???   INV  00 */ 0x000000,
/*11db: ???   INV  00 */ 0x000000,
/*11dc: ???   INV  00 */ 0x000000,
/*11dd: ???   INV  00 */ 0x000000,
/*11de: ???   INV  00 */ 0x000000,
/*11df: ???   INV  00 */ 0x000000,
/*11e0: ???   INV  00 */ 0x000000,
/*11e1: ???   INV  00 */ 0x000000,
/*11e2: ???   INV  00 */ 0x000000,
/*11e3: ???   INV  00 */ 0x000000,
/*11e4: ???   INV  00 */ 0x000000,
/*11e5: ???   INV  00 */ 0x000000,
/*11e6: ???   INV  00 */ 0x000000,
/*11e7: ???   INV  00 */ 0x000000,
/*11e8: ???   INV  00 */ 0x000000,
/*11e9: ???   INV  00 */ 0x000000,
/*11ea: ???   INV  00 */ 0x000000,
/*11eb: ???   INV  00 */ 0x000000,
/*11ec: ???   INV  00 */ 0x000000,
/*11ed: ???   INV  00 */ 0x000000,
/*11ee: ???   INV  00 */ 0x000000,
/*11ef: ???   INV  00 */ 0x000000,
/*11f0: ???   INV  00 */ 0x000000,
/*11f1: ???   INV  00 */ 0x000000,
/*11f2: ???   INV  00 */ 0x000000,
/*11f3: ???   INV  00 */ 0x000000,
/*11f4: ???   INV  00 */ 0x000000,
/*11f5: ???   INV  00 */ 0x000000,
/*11f6: ???   INV  00 */ 0x000000,
/*11f7: ???   INV  00 */ 0x000000,
/*11f8: ???   INV  00 */ 0x000000,
/*11f9: ???   INV  00 */ 0x000000,
/*11fa: ???   INV  00 */ 0x000000,
/*11fb: ???   INV  00 */ 0x000000,
/*11fc: ???   INV  00 */ 0x000000,
/*11fd: ???   INV  00 */ 0x000000,
/*11fe: ???   INV  00 */ 0x000000,
/*11ff: ???   INV  00 */ 0x000000};

#define GETBYTE i=Mgetc(pc)&0x00ff;pc+=1;sprintf(w,"%02X",i);strcat(hexa,w)
#define GETWORD i=((Mgetc(pc)&0xff)<<8)|(Mgetc(pc+1)&0xff);pc+=2;sprintf(w,"%04X",i);strcat(hexa,w)

//Creation d'une liste de registres //////////////////////////////////////////
void Reglist(int type)
{
// string: pointeur vers la chaine de retour de la liste de registres
// type: 0=EXG-TFR 1=PSHS 2=PULS 3=PSHU 4=PULU
 int i, c;
 const char s[8][4] = {"PC,","U,","Y,","X,","DP,","B,","A,","CC,"};
 const char u[8][4] = {"PC,","S,","Y,","X,","DP,","B,","A,","CC,"};
 const char r[16][4] = {"D,","X,","Y,","U,","S,","PC,","?,","?,",
                        "A,","B,","CC,","DP,","?,","?,","?,","?,"};
 param[0] = 0;
 c = Mgetc(pc++) & 0xff;   
 sprintf(w, "%02X", c); strcat(hexa, w); 
 switch(type)
 {
  case 0: sprintf(param, "%s%s", r[(c & 0xf0) >> 4], r[c & 0x0f]); break; 
  case 1: for(i = 0; i < 8; i++)  if(c & (128>>i)) strcat(param, s[i]); break;
  case 2: for(i = 7; i >= 0; i--) if(c & (128>>i)) strcat(param, s[i]); break;
  case 3: for(i = 0; i < 8; i++)  if(c & (128>>i)) strcat(param, u[i]); break;
  case 4: for(i = 7; i >= 0; i--) if(c & (128>>i)) strcat(param, u[i]); break;
 }
 if(strlen(param) > 0) param[strlen(param) - 1] = 0;
 if(type == 0) return;
 if(c & 0x80) n += 2;
 if(c & 0x40) n += 2;
 if(c & 0x20) n += 2;
 if(c & 0x10) n += 2;
 if(c & 0x08) n += 1;
 if(c & 0x04) n += 1;
 if(c & 0x02) n += 1;
 if(c & 0x01) n += 1;
}

//Decodage de l'adressage indexe /////////////////////////////////////////////
void Indexed()
{
 //string est la ligne desassemblee
 //operande est le champ des operandes
 //a est l'adresse en memoire du premier parametre de l'instruction indexee 
 //retour: adresse de l'instruction suivante  
 int i, x;
 char reg;
 //selection du registre concerne
 GETBYTE; x = i;
 switch(x & 0x60)
 {
  case 0x00: reg = 'X'; break;
  case 0x20: reg = 'Y'; break;
  case 0x40: reg = 'U'; break;
  case 0x60: reg = 'S'; break;
  default : reg = ' ';
 }
 switch(x & 0x9f)
 {
  case 0x80: n = 2; sprintf(param, ",%c+", reg); break;
  case 0x81: n = 3; sprintf(param, ",%c++", reg); break;
  case 0x82: n = 2; sprintf(param, ",-%c", reg); break;
  case 0x83: n = 3; sprintf(param, ",--%c", reg); break;
  case 0x84: n = 0; sprintf(param, ",%c", reg); break;
  case 0x85: n = 1; sprintf(param, "B,%c", reg); break;
  case 0x86: n = 1; sprintf(param, "A,%c", reg); break;
  case 0x87: n = 0; sprintf(param, "invalid"); break;
  case 0x88: n = 1; GETBYTE; sprintf(param, "$%02X,%c", i, reg); break;
  case 0x89: n = 4; GETWORD; sprintf(param, "$%04X,%c", i, reg); break;
  case 0x8a: n = 0; sprintf(param, "invalid"); break;
  case 0x8b: n = 4; sprintf(param, "D,%c", reg); break;
  case 0x8c: n = 1; GETBYTE; i = pc + (char)i; i &= 0xffff;
                    sprintf(param, "$%04X,%s", i, "PCR"); break;
  case 0x8d: n = 5; GETWORD; i = pc + (short)i; i &= 0xffff;
                    sprintf(param, "$%04X,%s", i, "PCR"); break;
  case 0x8e: n = 0; sprintf(param, "invalid"); break;
  case 0x8f: n = 0; sprintf(param, "invalid"); break;
  case 0x90: n = 3; sprintf(param, "invalid"); break;
  case 0x91: n = 6; sprintf(param, "[,%c++]", reg); break;
  case 0x92: n = 3; sprintf(param, "invalid"); break;
  case 0x93: n = 6; sprintf(param, "[,--%c]", reg); break;
  case 0x94: n = 3; sprintf(param, "[,%c]", reg); break;
  case 0x95: n = 4; sprintf(param, "[B,%c]", reg); break;
  case 0x96: n = 4; sprintf(param, "[A,%c]", reg); break;
  case 0x97: n = 3; sprintf(param, "invalid"); break;
  case 0x98: n = 4; GETBYTE; sprintf(param, "[$%02X,%c]", i, reg); break;
  case 0x99: n = 7; GETWORD; sprintf(param, "[$%04X,%c]", i, reg); break;
  case 0x9a: n = 3; sprintf(param, "invalid"); break;
  case 0x9b: n = 7; sprintf(param, "[D,%c]", reg); break;
  case 0x9c: n = 4; GETBYTE; i = pc + (char)i; i &= 0xffff;
                    sprintf(param, "[$%04X,%s]", i, "PCR"); break;
  case 0x9d: n = 8; GETWORD; i = pc + (short)i; i &= 0xffff;
                    sprintf(param, "[$%04X,%s]", i, "PCR"); break;
  case 0x9e: n = 3; sprintf(param, "invalid"); break;
  case 0x9f: n = 5; GETWORD; sprintf(param, "[$%04X]", i); break;
  default:   n = 1; if(!(x & 0x10)) sprintf(param, "$%02X,%c", x & 0x0f, reg);
                    else sprintf(param, "-$%02X,%c", 16 - (x & 0x0f), reg);
  //Assumes 0x84 for invalid bytes 0x87 0x8a 0x8e 0x8f
  //Assumes 0x94 for invalid bytes 0x90 0x92 0x97 0x9a 0x9e
 }
}

// Desassembly ///////////////////////////////////////////////////////////////
int Dasm6809(char *string, int a)
{// string = pointeur vers une chaine recevant la ligne desassemblee
 //      a = adresse des caracteres a desassembler
 // retour = adresse de l'instruction suivante
 int i, precode, code;
 //initialisations
 pc = a; //adresse de desassemblage
 n = 0;  //initialisation du nombre de cycles
 sprintf(hexa, "%04X ", a);
 //lecture du code instruction
 GETBYTE; code = i; precode = 0;
 if(code == 0x10) {precode = 0x100; i = Mgetc(pc) & 0xff;}           
 if(code == 0x11) {precode = 0x200; i = Mgetc(pc) & 0xff;}           
 if(precode > 0) if((i != 0x10) && (i != 0x11)) {GETBYTE; code = precode | i;}   
 //decodage de l'instruction
 switch(instruction[code] & 0xff00)
 {
  case 0x0100 /*INH */:  param[0] = 0; break;   
  case 0x0200 /*DIR */:  GETBYTE; sprintf(param, "/$%02X", i); break;   
  case 0x0300 /*IMM1*/: GETBYTE; sprintf(param, "#$%02X", i); break;   
  case 0x0400 /*IMM2*/: GETWORD; sprintf(param, "#$%04X", i); break;   
  case 0x0500 /*REL1*/: GETBYTE; sprintf(param, "$%04X", (pc + (char)i) & 0xffff); break;   
  case 0x0600 /*REL2*/: GETWORD; sprintf(param, "$%04X", (pc + (short)i) & 0xffff); break;   
  case 0x0700 /*IND */:  Indexed(); break;   
  case 0x0800 /*EXT */:  GETWORD; sprintf(param, "$%04X", i); break;   
  case 0x0a00 /*REG0*/: Reglist(0); break;   
  case 0x0b00 /*REG1*/: Reglist(1); break;   
  case 0x0c00 /*REG2*/: Reglist(2); break;   
  case 0x0d00 /*REG3*/: Reglist(3); break;   
  case 0x0e00 /*REG4*/: Reglist(4); break;   
  default:   param[0] = 0; break;
 }     
 //composition de la ligne desassemblee
 strcpy(string, hexa);
 for(i = strlen(string); i < 16; i++) strcat(string, " ");
 strcat(string, mnemonique[instruction[code] >> 16]); 
 for(i = strlen(string); i < 23; i++) strcat(string, " ");
 strcat(string, param); 
 for(i = strlen(string); i < 43; i++) strcat(string, " ");
 n += instruction[code] & 0xff;
 if(n == 0) strcpy(w, "?");
 if(n > 0) sprintf(w, "%i", n);
 if(n > 50) sprintf(w, "%i/%i", n / 10, n - 10 * (n / 10));
 strcat(string, w);
 //retour adresse prochaine instruction
 return(pc); 
}

