//////////////////////////////////////////////////////////////////////////////
// DCMO5BOUTONS.C - dessin des boutons et des touches du clavier
// Author   : Daniel Coulom - danielcoulom@gmail.com
// Web site : http://dcmo5.free.fr
// Created  : Novembre 2007
// Last updated : 2008-10-24
//
// This file is part of DCMO5 v11.
// 
// DCMO5 v11 is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// DCMO5 v11 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with DCMO5 v11.  If not, see <http://www.gnu.org/licenses/>.
//
//////////////////////////////////////////////////////////////////////////////

#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h> 
#include <stdio.h>
#include <string.h>
#include "dcmo5.h"

//surfaces des boutons relaches et enfonces
SDL_Surface *buttonsurfaceup[BOUTON_MAX][LANGUAGE_MAX];    //boutons, langages
SDL_Surface *buttonsurfacedown[BOUTON_MAX][LANGUAGE_MAX];  //boutons, langages

#if SDL_BYTEORDER == SDL_BIG_ENDIAN
const int rmask = 0xff000000, gmask = 0x00ff0000;
const int bmask = 0x0000ff00, amask = 0x000000ff;
const int blanc = 0xffffffff, noir  = 0x000000ff;
const int bleu  = 0x202060ff, grist = 0xe0e0e0e4;
const int gris0 = 0xe0e0e0ff;
const int gris1 = 0xd0d0d0ff;
const int gris2 = 0x808080ff;
const int gris3 = 0x606060ff;
#else
const int rmask = 0x000000ff, gmask = 0x0000ff00;
const int bmask = 0x00ff0000, amask = 0xff000000;
const int blanc = 0xffffffff, noir  = 0xff000000;
const int bleu  = 0xff602020, grist = 0xe4e0e0e0;
const int gris0 = 0xffe0e0e0;
const int gris1 = 0xffd0d0d0;
const int gris2 = 0xff808080;
const int gris3 = 0xff606060;
#endif
 
const button bouton[BOUTON_MAX] = {
{"[X]",       16, 14}, //00 
{"[prev]",    16,  8}, //01 
{"[next]",    16,  8}, //02 
{"",           0,  0}, //03 
{"",           0,  0}, //04 
{"",           0,  0}, //05
{"",           0,  0}, //06
{"",           0,  0}, //07
{"Options",   50, 16}, //08
{"?",         15, 16}, //09
{"",          10, 10}, //10
{"k7",        22, 16}, //11
{"fd",        22, 16}, //12
{"memo",      50, 16}, //13
{"{44}",      70, 20}, //14 
{"{45}",      70, 20}, //15
{"{36}",     214, 20}, //16
{"{37}",     214, 20}, //17
{"{38}",     214, 20}, //18
{"Action",    44, 18}, //19
{"N",         24, 24}, //0x00 
{"EFF",       24, 24}, //0x01 
{"J",         24, 24}, //0x02 
{"H",         24, 24}, //0x03 
{"U",         24, 24}, //0x04 
{"Y",         24, 24}, //0x05 
{"7 '",       24, 24}, //0x06 
{"6 &",       24, 24}, //0x07 
{" , <",      24, 24}, //0x08 
{"INS",       24, 24}, //0x09 
{"K",         24, 24}, //0x0a 
{"G",         24, 24}, //0x0b 
{"I",         24, 24}, //0x0c 
{"T",         24, 24}, //0x0d 
{"8 (",       24, 24}, //0x0e 
{"5 %",       24, 24}, //0x0f 
{" . >",      24, 24}, //0x10 
{"[retour]",  24, 24}, //0x11 
{"L",         24, 24}, //0x12  
{"F",         24, 24}, //0x13 
{"O",         24, 24}, //0x14 
{"R",         24, 24}, //0x15 
{"9 )",       24, 24}, //0x16 
{"4 $",       24, 24}, //0x17 
{"@ ^",       24, 24}, //0x18 
{"[droite]",  24, 24}, //0x19 
{"M",         24, 24}, //0x1a 
{"D",         24, 24}, //0x1b 
{"P",         24, 24}, //0x1c 
{"E",         24, 24}, //0x1d 
{"0 `",       24, 24}, //0x1e 
{"3 #",       24, 24}, //0x1f 
{"ESPACE",   108, 24}, //0x20 
{"[bas]",     24, 24}, //0x21 
{"B",         24, 24}, //0x22 
{"S",         24, 24}, //0x23 
{" / ?",      24, 24}, //0x24 
{"Z",         24, 24}, //0x25 
{" - =",      24, 24}, //0x26 
{"2 \"",      24, 24}, //0x27 
{"X",         24, 24}, //0x28 
{"[gauche]",  24, 24}, //0x29 
{"V",         24, 24}, //0x2a 
{"Q",         24, 24}, //0x2b 
{" * :",      24, 24}, //0x2c 
{"A",         24, 24}, //0x2d  
{" + ;",      24, 24}, //0x2e 
{"1 !",       24, 24}, //0x2f 
{"W",         24, 24}, //0x30 
{"[haut]",    24, 24}, //0x31 
{"C",         24, 24}, //0x32 
{"RAZ",       24, 24}, //0x33 
{"ENT",       40, 24}, //0x34 
{"CNT",       24, 24}, //0x35 
{"ACC",       24, 24}, //0x36 
{"STP",       24, 24}, //0x37 
{"SHIFT",     52, 24}, //0x38 
{"BASIC",     52, 24}  //0x39 
};
 
const dialogbutton closebutton = {-21, 5, 0};

const dialogbutton statusbutton[STATUSBUTTON_MAX] = {
{ 56,  2, 11}, //0x00 "k7",       
{186,  2, 12}, //0x01 "fd",       
{316,  2, 13}, //0x02 "memo",     
{-69,  2,  8}, //0x03 "Options",  
{-17,  2,  9}  //0x04 "?",        
};

const dialogbutton optionbutton[OPTIONBUTTON_MAX] = {
{208,  32,  1}, //0x00 "[prev]",  
{208,  40,  2}, //0x01 "[next]",  
{208,  52,  1}, //0x02 "[prev]",  
{208,  60,  2}, //0x03 "[next]",  
{208,  72,  1}, //0x04 "[prev]",  
{208,  80,  2}, //0x05 "[next]", 
{208,  92,  1}, //0x06 "[prev]",  
{208, 100,  2}, //0x07 "[next]", 
{208, 112,  1}, //0x08 "[prev]",  
{208, 120,  2}, //0x09 "[next]", 
{208, 132,  1}, //0x0a "[prev]",  
{208, 140,  2}, //0x0b "[next]", 
{ 10, 160, 18}  //0x0c "defaut", 
};

const dialogbutton desassbutton[DESASSBUTTON_MAX] = {
{10, 30,  1}, //0x00 "[prev]",   
{10, 54,  2}, //0x01 "[next]",   
{28, 30,  1}, //0x02 "[prev]",   
{28, 54,  2}, //0x03 "[next]",   
{46, 30,  1}, //0x04 "[prev]",   
{46, 54,  2}, //0x05 "[next]",  
{64, 30,  1}, //0x06 "[prev]",   
{64, 54,  2}, //0x07 "[next]",  
{10, 70, 14}, //0x08 "[debut]", 
{10, 95, 15}  //0x09 "[suite]", 
};
 
const dialogbutton keyboardbutton[KEYBOARDBUTTON_MAX] = {
{222, 114, 20}, //0x00 "N",         
{418, 114, 21}, //0x01 "EFF",       
{234,  86, 22}, //0x02 "J",         
{206,  86, 23}, //0x03 "H",         
{222,  58, 24}, //0x04 "U",         
{194,  58, 25}, //0x05 "Y",         
{206,  30, 26}, //0x06 "7 '",       
{178,  30, 27}, //0x07 "6 &",       
{250, 114, 28}, //0x08 " , <",      
{390, 114, 29}, //0x09 "INS",       
{262,  86, 30}, //0x0a "K",         
{178,  86, 31}, //0x0b "G",         
{250,  58, 32}, //0x0c "I",         
{166,  58, 33}, //0x0d "T",         
{234,  30, 34}, //0x0e "8 (",       
{150,  30, 35}, //0x0f "5 %",       
{278, 114, 36}, //0x10 " . >",      
{ 38,  86, 37}, //0x11 "[retour]",  
{290,  86, 38}, //0x12 "L",          
{150,  86, 39}, //0x13 "F",         
{278,  58, 40}, //0x14 "O",         
{138,  58, 41}, //0x15 "R",         
{262,  30, 42}, //0x16 "9 )",       
{122,  30, 43}, //0x17 "4 $",       
{306, 114, 44}, //0x18 "@ ^",       
{418,  58, 45}, //0x19 "[droite]",  
{318,  86, 46}, //0x1a "M",         
{122,  86, 47}, //0x1b "D",         
{306,  58, 48}, //0x1c "P",         
{110,  58, 49}, //0x1d "E",         
{290,  30, 50}, //0x1e "0 `",       
{ 94,  30, 51}, //0x1f "3 #",       
{150, 142, 52}, //0x20 "ESPACE",    
{402,  86, 53}, //0x21 "[bas]",     
{194, 114, 54}, //0x22 "B",         
{ 94,  86, 55}, //0x23 "S",         
{334,  58, 56}, //0x24 " / ?",      
{ 82,  58, 57}, //0x25 "Z",         
{318,  30, 58}, //0x26 " - =",      
{ 66,  30, 59}, //0x27 "2 \"",      
{110, 114, 60}, //0x28 "X",         
{390,  58, 61}, //0x29 "[gauche]",  
{166, 114, 62}, //0x2a "V",         
{ 66,  86, 63}, //0x2b "Q",         
{362,  58, 64}, //0x2c " * :",      
{ 54,  58, 65}, //0x2d "A",          
{346,  30, 66}, //0x2e " + ;",      
{ 38,  30, 67}, //0x2f "1 !",       
{ 82, 114, 68}, //0x30 "W",         
{402,  30, 69}, //0x31 "[haut]",    
{138, 114, 70}, //0x32 "C",         
{ 10,  86, 71}, //0x33 "RAZ",       
{346,  86, 72}, //0x34 "ENT",       
{ 10,  58, 73}, //0x35 "CNT",       
{374,  30, 74}, //0x36 "ACC",       
{ 10,  30, 75}, //0x37 "STP",       
{ 10, 114, 76}, //0x38 "SHIFT",     
{334, 114, 77}, //0x39 "BASIC",     
{ 10, 209, 16}, //0x00 "[defaut]",  
{228, 209, 17}  //0x01 "[sauver]",  
};
 
const dialogbutton joystickbutton[JOYSTICKBUTTON_MAX] = {
{135,  30, 69}, //0x00 "[0] haut",    
{135,  80, 53}, //0x01 "[0] bas",     
{110,  55, 61}, //0x02 "[0] gauche",  
{160,  55, 45}, //0x03 "[0] droite",  
{293,  30, 69}, //0x04 "[1] haut",    
{293,  80, 53}, //0x05 "[1] bas",     
{268,  55, 61}, //0x06 "[1] gauche",  
{318,  55, 45}, //0x07 "[1] droite",  
{123, 112, 19}, //0x08 "[0] action",  
{281, 112, 19}, //0x09 "[1] action",  
{ 10, 209, 16}, //0x00 "[defaut]",    
{228, 209, 17}  //0x01 "[sauver]",    
};

extern SDL_Surface *screen;    //surface d'affichage de l'ecran
extern SDL_Surface *textbox;   //surface d'affichage de texte
extern TTF_Font *vera11;       //police d'ecriture taille 11
extern TTF_Font *vera9;        //police d'ecriture taille 9
extern SDL_Color textnoir;     //couleur d'ecriture noir  
extern char *msg[][MSG_MAX];   //messages en plusieurs langues

//Dessin signes previous/next sur une surface /////////////////////////////////
void Drawnext(SDL_Surface *s, int orientation)
{
 SDL_Rect r;
 r.x = 6; r.y = 3; r.w = 3; r.h = 1; SDL_FillRect(s, &r, noir);
 r.x += 1; r.y += orientation; r.w = 1; SDL_FillRect(s, &r, noir);
}

//Dessin d'une croix sur une surface //////////////////////////////////////////
void Drawx(SDL_Surface *s)
{
 SDL_Rect r;
 int i;
 r.x = 4; r.y = 3; r.w = 2; r.h =1;
 for(i = 0; i < 7; i++) {SDL_FillRect(s, &r, noir); r.x++; r.y++;}
 r.x = 10; r.y = 3;
 for(i = 0; i < 7; i++) {SDL_FillRect(s, &r, noir); r.x--; r.y++;}
}

//Dessin d'une fleche sur une surface /////////////////////////////////////////
void Drawarrow(SDL_Surface *s, int orientation)
{
 SDL_Rect r;
 Sint16 *x, *y;
 Uint16 *w, *h; 
 int i, j, k, x0, y0, sens;
 char arrow[] = {0, 0, 1, 0, 1, 1, 1, 1, 0};

 //orientation: 0=gauche 1=droite 2=haut 3=bas 4=home
 if(orientation & 1) {sens = -1; x0 = 18; y0 = 11;}
                else {sens = 1; x0 = 3; y0 = 11;}
 if(orientation & 2) {x = &r.y; y = &r.x; w = &r.h; h = &r.w;}
                else {x = &r.x; y = &r.y; w = &r.w; h = &r.h;}
 if(orientation & 4) y0 -= 3;
 //dessin de la fleche
 k = 0; *w = 2; *h = 1;
 for(j = 0; j < 3; j++) for(i = 0; i < 3; i++)
 {
  if(arrow[k++] == 0) continue;
  *x = x0 + (2 + i + i) * sens;
  *y = y0 + (j - 3) * sens; SDL_FillRect(s, &r, gris3); 
  *y = y0 - (j - 3) * sens; SDL_FillRect(s, &r, gris3); 
 }      
 *x = x0 - 13 * (1 - sens) / 2; *y = y0; *w = 15; SDL_FillRect(s, &r, noir);
 if(!(orientation & 4)) return;
 *y += 8; SDL_FillRect(s, &r, noir);
 *w = 1; *h = 9; *y -= 8; *x += 15; SDL_FillRect(s, &r, noir);
}

//Initialisation image de fond d'un bouton relache ////////////////////////////
void Initbuttonup(SDL_Surface *s, int w, int h)
{
 SDL_Rect rect;
 rect.x = 0; rect.y = 0; rect.w = w;  rect.h = h;
 SDL_FillRect(s, &rect, gris1); 
 rect.x = 1; rect.y = 1; rect.w = w - 1;  rect.h = h - 1;
 SDL_FillRect(s, &rect, noir); 
 rect.x = 0; rect.y = 0; rect.w = w - 1;  rect.h = h - 1;
 SDL_FillRect(s, &rect, blanc); 
 rect.x = 1; rect.y = 1; rect.w = w - 2; rect.h = h - 2;
 SDL_FillRect(s, &rect, gris2); 
 rect.x = 1; rect.y = 1; rect.w = w - 3; rect.h = h - 3;
 SDL_FillRect(s, &rect, gris1);
}     

//Initialisation image de fond d'un bouton enfonce ////////////////////////////
void Initbuttondown(SDL_Surface *s, int w, int h)
{
 SDL_Rect rect;
 rect.x = 0; rect.y = 0; rect.w = w;  rect.h = h;
 SDL_FillRect(s, &rect, noir); 
 rect.x = 1; rect.y = 1; rect.w = w - 1; rect.h = h - 1;
 SDL_FillRect(s, &rect, gris2); 
 rect.x = 2; rect.y = 2; rect.w = w - 2;  rect.h = h - 2;
 SDL_FillRect(s, &rect, blanc); 
 rect.x = 2; rect.y = 2; rect.w = w - 3; rect.h = h - 3;
 SDL_FillRect(s, &rect, gris1);
}     

//Arrondir les coins des touches relachees ////////////////////////////////////
void Roundcornerup(SDL_Surface *s, int w, int h)
{
 SDL_Rect rect;
 rect.w = 1;  rect.h = 1;
 rect.x = 0;     rect.y = 0;     SDL_FillRect(s, &rect, gris0); 
 rect.x = w - 2; rect.y = 0;     SDL_FillRect(s, &rect, gris0);
 rect.x = w - 1; rect.y = 0;     SDL_FillRect(s, &rect, gris0);
 rect.x = 1;     rect.y = 1;     SDL_FillRect(s, &rect, blanc);
 rect.x = w - 3; rect.y = 1;     SDL_FillRect(s, &rect, blanc);
 rect.x = w - 1; rect.y = 1;     SDL_FillRect(s, &rect, gris0);
 rect.x = 1;     rect.y = h - 3; SDL_FillRect(s, &rect, blanc);
 rect.x = w - 3; rect.y = h - 3; SDL_FillRect(s, &rect, gris2);
 rect.x = 0;     rect.y = h - 2; SDL_FillRect(s, &rect, gris0);
 rect.x = w - 2; rect.y = h - 2; SDL_FillRect(s, &rect, noir);
 rect.x = 0;     rect.y = h - 1; SDL_FillRect(s, &rect, gris0);
 rect.x = 1;     rect.y = h - 1; SDL_FillRect(s, &rect, gris0);
 rect.x = w - 1; rect.y = h - 1; SDL_FillRect(s, &rect, gris0);
}     

//Arrondir les coins des touches enfoncees ////////////////////////////////////
void Roundcornerdown(SDL_Surface *s, int w, int h)
{
 SDL_Rect rect;
 rect.w = 1;  rect.h = 1;
 rect.x = 0;     rect.y = 0;     SDL_FillRect(s, &rect, gris0); 
 rect.x = w - 1; rect.y = 0;     SDL_FillRect(s, &rect, gris0); 
 rect.x = 1;     rect.y = 1;     SDL_FillRect(s, &rect, noir); 
 rect.x = 2;     rect.y = 2;     SDL_FillRect(s, &rect, gris2); 
 rect.x = w - 2; rect.y = h - 2; SDL_FillRect(s, &rect, blanc); 
 rect.x = 0;     rect.y = h - 1; SDL_FillRect(s, &rect, gris0); 
 rect.x = w - 1; rect.y = h - 1; SDL_FillRect(s, &rect, gris0); 
}     

//Initialisation des surfaces des boutons ////////////////////////////////////
void Initbuttonsurfaces()
{
 SDL_Rect rect;
 int i, j, w, h;
 const char *string; 
 for(i = 0; i < BOUTON_MAX; i++)
 {
  if(bouton[i].name[0] == 0) continue; //bouton non cr 
  w = bouton[i].w;
  h = bouton[i].h;
  //creation du bouton dans chaque langue
  for(j = 0; j < LANGUAGE_MAX; j++)
  {
   string = bouton[i].name;
   //boutons non traduits     
   if(j > 0) if(string[0] != '{')
   {
    buttonsurfaceup[i][j] = buttonsurfaceup[i][0];
    buttonsurfacedown[i][j] = buttonsurfacedown[i][0];
    continue;
   }
   //creation de la surface et du fond
   buttonsurfaceup[i][j] = SDL_CreateRGBSurface
    (SDL_SWSURFACE | SDL_DOUBLEBUF, w, h, 32, rmask, gmask, bmask, amask);
   buttonsurfacedown[i][j] = SDL_CreateRGBSurface
    (SDL_SWSURFACE | SDL_DOUBLEBUF, w, h, 32, rmask, gmask, bmask, amask);
   Initbuttonup(buttonsurfaceup[i][j], w, h);
   if(i > 18) Roundcornerup(buttonsurfaceup[i][j], w, h);
   Initbuttondown(buttonsurfacedown[i][j], w, h);
   if(i > 18) Roundcornerdown(buttonsurfacedown[i][j], w, h);
   //ajout d'un bitmap
   if(string[0] == '[')
   {
    switch(string[1])
    {
     case 'p': Drawnext(buttonsurfaceup[i][j], -1);                                           
               Drawnext(buttonsurfacedown[i][j], -1); break;                                           
     case 'n': Drawnext(buttonsurfaceup[i][j], 1);                                           
               Drawnext(buttonsurfacedown[i][j], 1); break;                                           
     case 'h': Drawarrow(buttonsurfaceup[i][j], 2);                                           
               Drawarrow(buttonsurfacedown[i][j], 2); break;                                           
     case 'b': Drawarrow(buttonsurfaceup[i][j], 3);                                           
               Drawarrow(buttonsurfacedown[i][j], 3); break;                                           
     case 'g': Drawarrow(buttonsurfaceup[i][j], 0);                                           
               Drawarrow(buttonsurfacedown[i][j], 0); break;                                           
     case 'd': Drawarrow(buttonsurfaceup[i][j], 1);                                           
               Drawarrow(buttonsurfacedown[i][j], 1); break;                                           
     case 'r': Drawarrow(buttonsurfaceup[i][j], 4);                                           
               Drawarrow(buttonsurfacedown[i][j], 4); break;                                           
     default : Drawx(buttonsurfaceup[i][j]);                                           
               Drawx(buttonsurfacedown[i][j]); break;                                           
    }                                            
    continue;                     
   } 
   //recherche du texte traduit en plusieurs langues
   if(string[0] == '{')
   {
    int imsg;                   
    imsg = strtol(string + 1, NULL, 10);
    if(imsg < 0) imsg = 0;
    if(imsg > MSG_MAX) imsg = 0;
    string = msg[j][imsg];
   } 
   //creation de l'image du texte
   SDL_FreeSurface(textbox);
   rect.x = (i > 19) ? 3 : 4;  //3 pour les touches, 4 pour les autres
   rect.y = (i > 19) ? 2 : 1;  //2 pour les touches, 1 pour les autres
   textbox = TTF_RenderText_Blended((i > 19) ? vera9 : vera11, string, textnoir);
   if(textbox == NULL) {SDL_error(25); return;}
   //application du texte sur la surface du bouton 
   if(SDL_BlitSurface(textbox, NULL, buttonsurfaceup[i][j], &rect) < 0) 

SDL_error(26);
   if(SDL_BlitSurface(textbox, NULL, buttonsurfacedown[i][j], &rect) < 0) 

SDL_error(27);
  }
 }
}     
