//////////////////////////////////////////////////////////////////////////////
// DCMO5KEYB.C - keyboard input and customization
// Author   : Daniel Coulom - danielcoulom@gmail.com
// Web site : http://dcmo5.free.fr
// Created  : December 2007
// Last updated : 2008-10-24
//
// This file is part of DCMO5 v11.
// 
// DCMO5 v11 is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// DCMO5 v11 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with DCMO5 v11.  If not, see <http://www.gnu.org/licenses/>.
//
//////////////////////////////////////////////////////////////////////////////

#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h> 
#include <stdio.h>
#include <string.h>
#ifdef WIN32
 #include <windows.h>
#endif 
#include "dcmo5.h"

//variables globales
char mo5keycode[256]; //scancode mo5 en fonction du scancode pc
char mo5joycode[256]; //numero bouton joystick en fonction du scancode pc
int lastkeycode;      //keycode derniere touche enfoncee
int lastkeysym;       //keysym derniere touche enfoncee

#ifdef WIN32
//scancode hardware du PC Windows pour chaque touche MO5
//(+ 0x40 pour les touches du pave numerique pour les distinguer des autres)
int pckeycode[KEYBOARDKEY_MAX]=
{
 0x31, //0x00 N
 0x53, //0x01 EFF
 0x24, //0x02 J
 0x23, //0x03 H
 0x16, //0x04 U
 0x15, //0x05 Y
 0x08, //0x06 7 '
 0x07, //0x07 6 &
 0x32, //0x08 , <
 0x52, //0x09 INS
 0x25, //0x0a K
 0x22, //0x0b G
 0x17, //0x0c I
 0x14, //0x0d T
 0x09, //0x0e 8 (
 0x06, //0x0f 5 %
 0x33, //0x10 . >
 0x47, //0x11 DEBUT
 0x26, //0x12 L 
 0x21, //0x13 F
 0x18, //0x14 O
 0x13, //0x15 R
 0x0a, //0x16 9 )
 0x05, //0x17 4 $
 0x34, //0x18 @ ^
 0x4d, //0x19 DROITE
 0x27, //0x1a M
 0x20, //0x1b D
 0x19, //0x1c P
 0x12, //0x1d E
 0x0b, //0x1e 0 `
 0x04, //0x1f 3 #
 0x39, //0x20 ESPACE
 0x50, //0x21 BAS
 0x30, //0x22 B
 0x1f, //0x23 S
 0x28, //0x24 / ?
 0x11, //0x25 Z
 0x0c, //0x26 - =
 0x03, //0x27 2 "
 0x2d, //0x28 X
 0x4b, //0x29 GAUCHE
 0x2f, //0x2a V
 0x1e, //0x2b Q
 0x2b, //0x2c * :
 0x10, //0x2d A 
 0x0d, //0x2e + ;
 0x02, //0x2f 1 !
 0x2c, //0x30 W
 0x48, //0x31 HAUT
 0x2e, //0x32 C
 0x4f, //0x33 RAZ
 0x1c, //0x34 ENTREE
 0x1d, //0x35 CNT
 0x1b, //0x36 ACC
 0x29, //0x37 STOP
 0x2a, //0x38 MAJUSCULE
 0x36  //0x39 BASIC
};

//scancode hardware du PC Windows pour chaque fonction des manettes MO5
//(+ 0x40 pour les touches du pave numerique pour les distinguer des autres)
int pcjoycode[JOYSTICKKEY_MAX]=
{
 0x8c, //0 manette 0 HAUT 
 0x90, //1 manette 0 BAS
 0x8f, //2 manette 0 GAUCHE
 0x91, //3 manette 0 DROITE 
 0x75, //4 manette 1 HAUT 
 0x88, //5 manette 1 BAS
 0x87, //6 manette 1 GAUCHE
 0x89, //7 manette 1 DROITE
 0x5c, //8 manette 0 ACTION
 0x8e  //9 manette 1 ACTION
};

#else

//scancode hardware du PC Linux pour chaque touche MO5
//(+ 0x40 pour les touches du pave numerique pour les distinguer des autres)
int pckeycode[KEYBOARDKEY_MAX]=
{
 0x39, //0x00 N
 //0x6b, //0x01 EFF
 0x77, //0x01 EFF
 0x2c, //0x02 J
 0x2b, //0x03 H
 0x1e, //0x04 U
 0x1d, //0x05 Y
 0x10, //0x06 7 '
 0x0f, //0x07 6 &
 0x3a, //0x08 , <
 //0x6a, //0x09 INS
 0x76, //0x09 INS
 0x2d, //0x0a K
 0x2a, //0x0b G
 0x1f, //0x0c I
 0x1c, //0x0d T
 0x11, //0x0e 8 (
 0x0e, //0x0f 5 %
 0x3b, //0x10 . >
 //0x61, //0x11 DEBUT
 0x6e, //0x11 DEBUT
 0x2e, //0x12 L 
 0x29, //0x13 F
 0x20, //0x14 O
 0x1b, //0x15 R
 0x12, //0x16 9 )
 0x0d, //0x17 4 $
 0x3c, //0x18 @ ^
 //0x66, //0x19 DROITE
 0x72, //0x19 DROITE
 0x2f, //0x1a M
 0x28, //0x1b D
 0x21, //0x1c P
 0x1a, //0x1d E
 0x13, //0x1e 0 `
 0x0c, //0x1f 3 #
 0x41, //0x20 ESPACE
 //0x68, //0x21 BAS
 0x74, //0x21 BAS
 0x38, //0x22 B
 0x27, //0x23 S
 0x30, //0x24 / ?
 0x19, //0x25 Z
 0x14, //0x26 - =
 0x0b, //0x27 2 "
 0x35, //0x28 X
 //0x64, //0x29 GAUCHE
 0x71, //0x29 GAUCHE
 0x37, //0x2a V
 0x26, //0x2b Q
 0x33, //0x2c * :
 0x18, //0x2d A 
 0x15, //0x2e + ;
 0x0a, //0x2f 1 !
 0x34, //0x30 W
 //0x62, //0x31 HAUT
 0x6F, //0x31 HAUT
 0x36, //0x32 C
 //0x67, //0x33 RAZ
 0x92, //0x33 RAZ
 0x24, //0x34 ENTREE
 0x25, //0x35 CNT
 0x23, //0x36 ACC
 0x31, //0x37 STOP
 0x32, //0x38 MAJUSCULE
 0x3e  //0x39 BASIC
};

//scancode hardware du PC Linux pour chaque fonction des manettes MO5
//(+ 0x40 pour les touches du pave numerique pour les distinguer des autres)
int pcjoycode[JOYSTICKKEY_MAX]=
{
 0x94, //0 manette 0 HAUT 
 0x98, //1 manette 0 BAS
 0x97, //2 manette 0 GAUCHE
 0x99, //3 manette 0 DROITE 
 //0xb0, //4 manette 1 HAUT 
 0xaa, //4 manette 1 HAUT 
 0x90, //5 manette 1 BAS
 0x8f, //6 manette 1 GAUCHE
 0x91, //7 manette 1 DROITE
 //0xac, //8 manette 0 ACTION
 0xa8, //8 manette 0 ACTION
 0x96  //9 manette 1 ACTION
};

#endif

//variables externes
extern int language;
extern int touche[];
extern int rmask, gmask, bmask, amask;
extern int dialog;             //0 ou nboite de dialogue affichee
extern SDL_Surface *screen;    //surface d'affichage de l'ecran
extern SDL_Surface *textbox;   //surface d'affichage de texte
extern SDL_Surface *dialogbox; //surface d'affichage dialogbox
extern SDL_Surface *buttonsurfaceup[][LANGUAGE_MAX]; 
extern SDL_Surface *buttonsurfacedown[][LANGUAGE_MAX]; 
extern char *msg[LANGUAGE_MAX][MSG_MAX]; //messages en plusieurs langues
extern button bouton[];
extern const dialogbutton keyboardbutton[];
extern const dialogbutton joystickbutton[];

//Emulation clavier : affichage du nom de la derniere touche pressee //////////
void Displaykey()
{
 SDL_Rect rect;
 int i;
 char string[80];
 //code et nom de la touche PC
 rect.x = 10; rect.y = 154; rect.w = 135; rect.h = 15;
 sprintf(string, "0x%02x = %s", lastkeycode, SDL_GetKeyName(lastkeysym));    
 Drawtextbox(dialogbox, string, rect, 1); 
 //code et nom de la touche MO5
 rect.x = 275; rect.y = 154; rect.w = 170;
 rect.h = 15;
 i = mo5keycode[lastkeycode & 0xff];
 sprintf(string, "%s : %s", msg[language][15],    
  (i < KEYBOARDKEY_MAX) ? bouton[keyboardbutton[i].n].name : msg[language][18]);
 Drawtextbox(dialogbox, string, rect, 1); 
}           

//Emulation manettes : affichage du nom de la derniere touche pressee /////////
void Displayjoy()
{
 SDL_Rect rect;
 int i;
 char string[80];
 //code et nom de la touche PC
 rect.x = 80; rect.y = 154; rect.w = 135; rect.h = 15;
 sprintf(string, "0x%02x = %s", lastkeycode, SDL_GetKeyName(lastkeysym));    
 Drawtextbox(dialogbox, string, rect, 1); 
 //code et nom de la fonction manette
 rect.x = 230; rect.y = 154; rect.w = 175; rect.h = 15;
 i = mo5joycode[lastkeycode & 0xff];
 sprintf(string, "%s : %s", msg[language][35],    
  (i < JOYSTICKKEY_MAX) ? bouton[joystickbutton[i].n].name : msg[language][18]);
 Drawtextbox(dialogbox, string, rect, 1); 
}           

//Cration de la boite de dialogue du clavier ////////////////////////////////
void Clavier()
{
 SDL_Rect rect;
 int i, n;
 char string[256];
 extern int pause6809;
 pause6809 = 1;
 lastkeycode = 0;
 lastkeysym = 0;
 Createdialogbox(454, 244);
 //titre
 rect.x = 10; rect.w = dialogbox->w - 32;
 rect.y = 5; rect.h = 15;
 Drawtextbox(dialogbox, msg[language][14], rect, 13);
 //message d'aide
 rect.x = 80; rect.y = 175; rect.w = 370;
 sprintf(string, "%s %s.", msg[language][16], msg[language][15]);
 Drawtextbox(dialogbox, string, rect, 4); 
 rect.y += 15;
 Drawtextbox(dialogbox, msg[language][17], rect, 4); 
 //dessin des boutons
 for(i = 0; i < KEYBOARDBUTTON_MAX; i++)
 {
  n = keyboardbutton[i].n;     
  rect.x = keyboardbutton[i].x;
  rect.y = keyboardbutton[i].y;
  SDL_BlitSurface(buttonsurfaceup[n][language], NULL, dialogbox, &rect);
 } 
 dialog = 3;     
 Displaykey(0);  //nom de la touche lastkey
 pause6809 = 0;
}     

//Cration de la boite de dialogue des manettes //////////////////////////////
void Manettes()
{
 SDL_Rect rect;
 int i, n;
 char string[256];
 extern int pause6809;
 pause6809 = 1;
 lastkeycode = 0;
 lastkeysym = 0;
 Createdialogbox(454, 244);
 //titre
 rect.x = 10; rect.w = dialogbox->w - 32;
 rect.y = 5; rect.h = 15;
 Drawtextbox(dialogbox, msg[language][34], rect, 13);
 //message d'aide
 rect.x = 80; rect.y = 175; rect.w = 370;
 sprintf(string, "%s %s.", msg[language][16], msg[language][35]);
 Drawtextbox(dialogbox, string, rect, 4); 
 rect.y += 15;
 Drawtextbox(dialogbox, msg[language][17], rect, 4); 
 //boutons et numeros des manettes
 for(i = 0; i < JOYSTICKBUTTON_MAX; i++)
 {
  n = joystickbutton[i].n;     
  rect.x = joystickbutton[i].x;
  rect.y = joystickbutton[i].y;
  SDL_BlitSurface(buttonsurfaceup[n][language], NULL, dialogbox, &rect);
 }            
 rect.x = 136; rect.y = 132; rect.w = 20;
 Drawtextbox(dialogbox, "[0]", rect, 4); 
 rect.x += 158;
 Drawtextbox(dialogbox, "[1]", rect, 4); 
 dialog = 4;
 Displayjoy(0); //nom de la fonction lastjoy
 pause6809 = 0;
}     

// Restauration de la configuration par defaut des touches ///////////////////
void Restorekeydefault()
{
 int i, j;
 for(i = 0; i < 256; i++)
 {
  for(j = 0; j < KEYBOARDKEY_MAX; j++) if(pckeycode[j] == i) break;
  mo5keycode[i] = j;     
 }
} 

// Restauration de la configuration par defaut des manettes //////////////////
void Restorejoydefault()
{
 int i, j;
 for(i = 0; i < 256; i++)
 {
  for(j = 0; j < JOYSTICKKEY_MAX; j++) if(pcjoycode[j] == i) break;
  mo5joycode[i] = j;     
 }
} 

// Sauvegarde de la configuration du clavier /////////////////////////////////
void Savekeyfile()
{
 extern FILE *fpi;
 fseek(fpi, 0x40, SEEK_SET);    
 fwrite(mo5keycode, 256, 1, fpi);    
} 

// Sauvegarde de la configuration des manettes ///////////////////////////////
void Savejoyfile()
{
 extern FILE *fpi;
 fseek(fpi, 0x140, SEEK_SET);    
 fwrite(mo5joycode, 256, 1, fpi);    
} 

//Traitement des clics boite de dialogue clavier /////////////////////////////
void Keyclick()
{
 SDL_Rect rect;
 int i, j, n, x, y;    
 extern SDL_Rect dialogrect;    //position dialogbox 
 extern int xmouse, ymouse;
 //recherche d'un clic bouton
 for(i = 0; i < KEYBOARDBUTTON_MAX; i++)
 {
  n = keyboardbutton[i].n;     
  x = dialogrect.x + keyboardbutton[i].x;     
  y = dialogrect.y + keyboardbutton[i].y;     
  if(xmouse > x) if(xmouse < (x + bouton[keyboardbutton[i].n].w))
  if(ymouse > y) if(ymouse < (y + bouton[keyboardbutton[i].n].h))
  break;
 }
 if(i >= KEYBOARDBUTTON_MAX) return;
 //dessin du bouton enfonce puis relache
 rect.x = keyboardbutton[i].x;
 rect.y = keyboardbutton[i].y;
 SDL_BlitSurface(buttonsurfacedown[n][language], NULL, dialogbox, &rect);
 Displayscreen(); SDL_Delay(200);
 SDL_BlitSurface(buttonsurfaceup[n][language], NULL, dialogbox, &rect);
 //boutons restauration et sauvegarde
 if(n == 16) {Restorekeydefault(); return;}
 if(n == 17) {Savekeyfile(); return;}
 //suppression de l'ancienne affectation de la touche           
 for(j = 0; j < 256; j++) if(mo5keycode[j] == i) mo5keycode[j] = KEYBOARDKEY_MAX;
 //ajout de la nouvelle affectation
 if(lastkeycode != 0) mo5keycode[lastkeycode & 0xff] = i; 
 Displaykey();
}

//Traitement des clics boite de dialogue manettes ////////////////////////////
void Joyclick()
{
 SDL_Rect rect;
 int i, j, n, x, y;    
 extern SDL_Rect dialogrect;    //position dialogbox 
 extern int xmouse, ymouse;
 //recherche d'un clic bouton
 for(i = 0; i < JOYSTICKBUTTON_MAX; i++)
 {
  n = joystickbutton[i].n;     
  x = dialogrect.x + joystickbutton[i].x;     
  y = dialogrect.y + joystickbutton[i].y;     
  if(xmouse > x) if(xmouse < (x + bouton[joystickbutton[i].n].w))
  if(ymouse > y) if(ymouse < (y + bouton[joystickbutton[i].n].h))
  break;
 }
 if(i >= JOYSTICKBUTTON_MAX) return;
 //dessin du bouton enfonce puis relache
 rect.x = joystickbutton[i].x;
 rect.y = joystickbutton[i].y;
 SDL_BlitSurface(buttonsurfacedown[n][language], NULL, dialogbox, &rect);
 Displayscreen(); SDL_Delay(200);
 SDL_BlitSurface(buttonsurfaceup[n][language], NULL, dialogbox, &rect);
 //touches restauration et sauvegarde
 if(n == 16) {Restorejoydefault(); return;}
 if(n == 17) {Savejoyfile(); return;}
 //suppression de l'ancienne affectation de la touche           
 for(j = 0; j < 256; j++) if(mo5joycode[j] == i) mo5joycode[j] = JOYSTICKKEY_MAX;
 //ajout de la nouvelle affectation
 if(lastkeycode != 0) mo5joycode[lastkeycode & 0xff] = i; 
 Displayjoy();
}

// Initialisation du clavier /////////////////////////////////////////////////
void Keyboardinit()
{
 int i, j; 
 extern FILE *fpi;   
 //initialisation des tables par defaut
 for(i = 0; i < 256; i++)
 {
  for(j = 0; j < KEYBOARDKEY_MAX; j++) if(pckeycode[j] == i) break;
  mo5keycode[i] = j;     
  for(j = 0; j < JOYSTICKKEY_MAX; j++) if(pcjoycode[j] == i) break;
  mo5joycode[i] = j;     
 }
 //recuperation des valeurs de dcmo5.ini (0x40=touches 0x140=joysticks) 
 fseek(fpi, 0, SEEK_END);
 i = (int)ftell(fpi); //Info(i);  
 if(i < 0x140) return;
 fseek(fpi, 0x40, SEEK_SET); j = fgetc(fpi); fseek(fpi, 0x40, SEEK_SET); 
 if(j == KEYBOARDKEY_MAX) fread(mo5keycode, 256, 1, fpi);
 if(i < 0x240) return;
 fseek(fpi, 0x140, SEEK_SET); j = fgetc(fpi); fseek(fpi, 0x140, SEEK_SET); 
 if(j == JOYSTICKKEY_MAX) fread(mo5joycode, 256, 1, fpi);    
}

// Key up ////////////////////////////////////////////////////////////////////
void Keyup(int keysym, int scancode)
{
 SDL_Rect rect;       
 int ijoy, ikey, n, keycode;
 //le scancode seul ne permet pas de distinguer le pave numerique
 //keycode = scancode + 0x40 pour le pave numerique
 //keycode = scancode pour toutes les autres touches
 keycode = scancode;
 if(keysym == 0x12c) keycode += 0x40;           //numlock
 if((keysym & 0xff0) == 0x100) keycode += 0x40; //autres touches numpad

 //emulation joystick
 ijoy = mo5joycode[keycode & 0xff];
 ikey = mo5keycode[keycode & 0xff];

 if(ijoy < JOYSTICKKEY_MAX)
 {
  Joysemul(ijoy, 0x80);
  if(dialog == 4)
  {
   n = joystickbutton[ijoy].n;
   rect.x = joystickbutton[ijoy].x;
   rect.y = joystickbutton[ijoy].y;
   SDL_BlitSurface(buttonsurfaceup[n][language], NULL, dialogbox, &rect);
  }
 }  

 //emulation clavier
 if(ikey < KEYBOARDKEY_MAX)
 {
  touche[ikey] = 0x80;
  //dessin de la touche relachee
  if(dialog == 3)
  {
   n = keyboardbutton[ikey].n;
   rect.x = keyboardbutton[ikey].x;
   rect.y = keyboardbutton[ikey].y;
   SDL_BlitSurface(buttonsurfaceup[n][language], NULL, dialogbox, &rect);
  }
 } 
} 

// Key down //////////////////////////////////////////////////////////////////
void Keydown(int keysym, int scancode)
{
 SDL_Rect rect;         
 int ijoy, ikey, n, keycode;
 extern int pause6809;
 //le scancode seul ne permet pas de distinguer le pave numerique
 //keycode = scancode + 0x40 pour le pave numerique
 //keycode = scancode pour toutes les autres touches
 keycode = scancode;
 if(keysym == 0x12c) keycode += 0x40;           //numlock
 if((keysym & 0xff0) == 0x100) keycode += 0x40; //autres touches numpad
 lastkeycode = keycode;
 lastkeysym = keysym;

 //essai (infructueux) de detection des deux touches shift simultanees 
 //quand une touche shift est enfonce, les mouvements de l'autre
 //ne sont pas detectes, et elle est toujours consideree comme relachee
 //idem pour les touches CTRL droit et gauche
 //difference pour les touches ALT : si la gauche est enfoncee, la droite
 //est detectee, mais pas l'inverse.
 //Remarque : ce comportement est observe dans Windows, mais pas dans Linux 

 //static int flag = 0;
 //Uint8 *keystate;
 //SDL_PumpEvents();
 //keystate = SDL_GetKeyState(NULL);
 //flag = 256 - flag;
 //Keyboard(flag + 2 * keystate[SDLK_LSHIFT] + keystate[SDLK_RSHIFT]); return;
 
 //SDL_Event event;
 //event.key.type = SDL_KEYUP;
 //event.key.state = SDL_RELEASED;
 //Keyboard(SDL_GetModState());
 //SDL_PushEvent(&event);
 //SDL_SetModState(0);
 
 //touches de raccourcis dcmo5
 if(keysym == SDLK_ESCAPE) {Initprog(); pause6809 = 0; return;}
 if(keysym == SDLK_PAUSE) {pause6809 = 1; return;}            
 //if(keycode == 0x01) {Initprog(); pause6809 = 0; return;}//touche ESC enfoncee
 //if(keycode == 0x45) {pause6809 = 1; return;}            //touche PAUSE enfoncee
 
 //la touche AltGr envoie 2 evenements keydown
 //le premier keysym = 0x132 (LCTRL)
 //le deuxieme keysym = 0x133 (RALT)
 //il faut appeler Keyup pour annuler le faux Keydown de LCTRL
 //if(keysym == 0x133) Keyup(0x132);
 if(keycode == 0x38) Keyup(0, 0x1d);
 pause6809 = 0;  //l'appui sur une touche arrete la pause 
 
 //emulation joystick
 ijoy = mo5joycode[keycode & 0xff]; //Info(i);
 ikey = mo5keycode[keycode & 0xff];
 if(dialog == 3) Displaykey();
 if(dialog == 4) Displayjoy();


 if(ijoy < JOYSTICKKEY_MAX)
 {
  Joysemul(ijoy, 0x00);
  if(dialog == 4)
  {
   n = joystickbutton[ijoy].n;
   rect.x = joystickbutton[ijoy].x;
   rect.y = joystickbutton[ijoy].y;
   SDL_BlitSurface(buttonsurfacedown[n][language], NULL, dialogbox, &rect);
  }
 } 
 
 //emulation clavier
 if(ikey < KEYBOARDKEY_MAX)
 {
  touche[ikey] = 0x00;
  if(dialog == 3)
  {
   n = keyboardbutton[ikey].n;
   rect.x = keyboardbutton[ikey].x;
   rect.y = keyboardbutton[ikey].y;
   SDL_BlitSurface(buttonsurfacedown[n][language], NULL, dialogbox, &rect);
  } 
 }
}     

// Test touches SHIFT, ALT et CTRL gauche et droite //////////////////////////
void Testshiftkey()
{
 //Dans Windows, si une touche majuscule droite ou gauche est enfoncee
 //l'evenement SDL_KEYDOWN n'est pas envoye par l'autre.
 //Il faut donc tester periodiquement leur etat (pour Flipper par exemple).
 //Malheureusement les fonctions SDL_GetModState et SDL_GetKeyState ne
 //marchent pas en cas d'appui simultane sur SHIFT, CTRL et ALT droit et gauche 
 //Cette routine utilise la fonction GetAsyncKeyState de windows pour
 //detecter la veritable position des touches.
 //Elle est appellee environ 20 fois par seconde par la fonction Playsound  
 
 //SDL_GetModState ne marche pas si les 2 touches sont enfoncees
 //int mode = SDL_GetModState();
 //extern int touche[]; //position touches MO5
 //touche[mo5key[SDLK_LSHIFT]] = ((mode & 1)) ? 0x00 : 0x80;     
 //touche[mo5key[SDLK_RSHIFT]] = ((mode & 2)) ? 0x00 : 0x80;     
 //sprintf(infos, "ModState=%04x", mode);
 //Printstring(infos, 0, 238, 7);
 //SDL_GetKeyState ne marche pas mieux 
 //Uint8 *keystate = SDL_GetKeyState(NULL);
 //sprintf(infos, "Left=%i    Right=%i",
 //        keystate[SDLK_LSHIFT], keystate[SDLK_RSHIFT]);
 //Printstring(infos, 100, 238, 7);

#ifdef WIN32
 static int lshift, rshift; //position des touches shift   
 //Remarque : VK_LSHIFT et VK_RSHIFT ne marchent pas avec Windows 98se
 //D'aprs Microsoft : Windows 95/98/Me: Windows 95 does not support
 //the left- and right-distinguishing constants. If you call
 //GetAsyncKeyState with these constants, the return value is zero.
 if(GetAsyncKeyState(VK_LSHIFT) < 0)
   {if(!lshift) {lshift = 1; Keydown(SDLK_LSHIFT, 0x2a);}}                              
   else {if(lshift) {lshift = 0; Keyup(SDLK_LSHIFT, 0x2a);}}
 if(GetAsyncKeyState(VK_RSHIFT) < 0)
   {if(!rshift) {rshift = 1; Keydown(SDLK_RSHIFT, 0x36);}}                              
   else {if(rshift) {rshift = 0; Keyup(SDLK_RSHIFT, 0x36);}}
#endif
} 
