//////////////////////////////////////////////////////////////////////////////
// DCMO5MAIN.C - dcmo5 v11 emulator main program
// Author   : Daniel Coulom - danielcoulom@gmail.com
// Web site : http://dcmo5.free.fr
// Created  : July 2006
// Last updated : 2008-10-24
//
// This file is part of DCMO5 v11.
// 
// DCMO5 v11 is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// DCMO5 v11 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with DCMO5 v11.  If not, see <http://www.gnu.org/licenses/>.
//
//////////////////////////////////////////////////////////////////////////////


#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h> 
#include "dcmo5.h"

// global variables //////////////////////////////////////////////////////////
const SDL_VideoInfo *videoinfo;
SDL_AudioSpec audio;
int pause6809;        //processor pause state
int report;           //nombre de milliemes de cycle a reporter
//LANGUAGE_MAX langues : 0=francais 1=anglais
//MSG_MAX messages par langue

char *msg[LANGUAGE_MAX][MSG_MAX]={
{
/*00*/ " DCMO5 v11 - Emulateur MO5 Thomson",
/*01*/ "A propos de DCMO5",
/*02*/ "Version : 11.2 - 25 octobre 2008",
/*03*/ "Auteur : Daniel Coulom",
/*04*/ "Site internet : http://dcmo5.free.fr",
/*05*/ " ",
/*06*/ "Ce programme est diffus SANS AUCUNE GARANTIE.",
/*07*/ "C'est un logiciel libre, et vous tes encourags  le",
/*08*/ "redistribuer sous certaines conditions.",
/*09*/ "Lire les dtails dans dcmo5v11-licence.txt",
/*10*/ "Erreur",
/*11*/ "Pas de cassette dans le lecteur",
/*12*/ "Fin de cassette, rembobinage automatique",
/*13*/ "Cassette protge en criture",
/*14*/ "Disposition du clavier",
/*15*/ "touche MO5 mule",
/*16*/ "Appuyez sur une touche pour voir la ",
/*17*/ "Cliquez sur l'image pour changer la correspondance.",
/*18*/ "aucune",
/*19*/ "",
/*20*/ "Raccourcis clavier :",
/*21*/ "- ECHAP= initialisation programme",
/*22*/ "- PAUSE= arrt processeur 6809",
/*23*/ " ",
/*24*/ "Lire la documentation pour d'autres informations...",
/*25*/ "",
/*26*/ "Vitesse d'affichage",
/*27*/ "Franais",
/*28*/ "Vitesse processeur",
/*29*/ "Paramtres",
/*30*/ "Clavier",
/*31*/ "Manettes",
/*32*/ "Dsassemblage",
/*33*/ "",
/*34*/ "Emulation des manettes",
/*35*/ "fonction mule",
/*36*/ "Rtablir la disposition standard",
/*37*/ "Sauver la disposition personnalise",
/*38*/ "Rtablir les valeurs par dfaut",
/*39*/ "[dcharger]",
/*40*/ "[suite...]",
/*41*/ "[retour au dbut...]",
/*42*/ "en lecture seule",
/*43*/ "en lecture/criture",
/*44*/ "Dbut",
/*45*/ "Suite",
/*46*/ "",
/*47*/ "",
/*48*/ "",
/*49*/ "dcmo5.rom incorrect ou non trouv"
},

{
/*00*/ " DCMO5 v11 - Thomson MO5 emulator",
/*01*/ "About DCMO5",
/*02*/ "Version : 11.2 - October 25, 2008",
/*03*/ "Author : Daniel Coulom",
/*04*/ "Web site : http://dcmo5.free.fr",
/*05*/ " ",
/*06*/ "This program comes with ABSOLUTELY NO WARRANTY.",
/*07*/ "This is free software, and you are welcome",
/*08*/ "to redistribute it under certain conditions.",
/*09*/ "More details in dcmo5v11-licence.txt",
/*10*/ "Error",
/*11*/ "No tape currently loaded",
/*12*/ "End of tape, automatic rewind",
/*13*/ "Tape write error",
/*14*/ "Keyboard layout",
/*15*/ "emulated MO5 key",
/*16*/ "Press a key to show the ",
/*17*/ "Click a button in the picture to make a change.",
/*18*/ "none",
/*19*/ "",
/*20*/ "Keyboard shortcuts :",
/*21*/ "- ESC= reset MO5 program",
/*22*/ "- PAUSE= stop 6809 processor",
/*23*/ " ",
/*24*/ "Please check the documentation for more information.",
/*25*/ "",
/*26*/ "Display speed",
/*27*/ "English",
/*28*/ "Processor speed",
/*29*/ "Settings",
/*30*/ "Keyboard",
/*31*/ "Joysticks",
/*32*/ "Disassembly",
/*33*/ "",
/*34*/ "Joystick emulation",
/*35*/ "emulated function",
/*36*/ "Restore standard layout",
/*37*/ "Save customized layout",
/*38*/ "Restore default values",
/*39*/ "[unload]",
/*40*/ "[next...]",
/*41*/ "[back to first...]",
/*42*/ "read only",
/*43*/ "read/write",
/*44*/ "First",
/*45*/ "Next",
/*46*/ "",
/*47*/ "",
/*48*/ "",
/*49*/ "dcmo5.rom not found or corrupted"
}};

unsigned char cursor[] = {
128,0,192,0,160,0,144,0,136,0,132,0,130,0,129,0,128,128,128,64,128,32,129,240,
137,0,152,128,164,128,196,64,130,64,2,32,1,32,1,16,0,176,0,192,0,0,0,0,128,0,
192,0,224,0,240,0,248,0,252,0,254,0,255,0,255,128,255,192,255,224,255,240,255,
0,231,128,199,128,131,192,3,192,3,224,1,224,1,240,0,240,0,192,0,0,0,0};

char dcmo5icon[]={0x42,0x4d,0x76,0x02,0x00,0x00,
0x00,0x00,0x00,0x00,0x76,0x00,0x00,0x00,0x28,0x00,0x00,0x00,0x20,0x00,0x00,0x00,
0x20,0x00,0x00,0x00,0x01,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x00,0x00,0x80,0x00,0x00,0x00,0x80,0x80,0x00,
0x80,0x00,0x00,0x00,0x80,0x00,0x80,0x00,0x80,0x80,0x00,0x00,0x80,0x80,0x80,0x00,
0xc0,0xc0,0xc0,0x00,0x00,0x00,0xff,0x00,0x00,0xff,0x00,0x00,0x00,0xff,0xff,0x00,
0xff,0x00,0x00,0x00,0xff,0x00,0xff,0x00,0xff,0xff,0xC8,0x00,0xff,0xff,0xff,0x00,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xcc,0xcc,0xcc,0xcc,0xcc,0xee,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xcc,0xcc,0xcc,0xcc,0xcc,0xee,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xee,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xee,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,0xee,0xee,0xee,0xee,0xee,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,0xee,0xee,0xee,0xee,0xee,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,0xee,0xee,0xee,0xee,0xee,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,
0xee,0xcc,0xee,0xee,0xee,0xee,0xcc,0xee,0xee,0xcc,0xcc,0xcc,0xcc,0xcc,0xee,0xee,
0xee,0xcc,0xee,0xee,0xee,0xee,0xcc,0xee,0xee,0xcc,0xcc,0xcc,0xcc,0xcc,0xee,0xee,
0xee,0xcc,0xee,0xee,0xee,0xee,0xcc,0xee,0xcc,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,
0xee,0xcc,0xee,0xee,0xee,0xee,0xcc,0xee,0xcc,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,
0xee,0xcc,0xee,0xee,0xee,0xee,0xcc,0xee,0xcc,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,
0xee,0xcc,0xee,0xee,0xee,0xee,0xcc,0xee,0xcc,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,
0xee,0xcc,0xee,0xcc,0xcc,0xee,0xcc,0xee,0xcc,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,
0xee,0xcc,0xee,0xcc,0xcc,0xee,0xcc,0xee,0xcc,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,
0xee,0xcc,0xcc,0xee,0xee,0xcc,0xcc,0xee,0xcc,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,
0xee,0xcc,0xcc,0xee,0xee,0xcc,0xcc,0xee,0xcc,0xee,0xee,0xee,0xee,0xee,0xcc,0xee,
0xee,0xcc,0xee,0xee,0xee,0xee,0xcc,0xee,0xee,0xcc,0xcc,0xcc,0xcc,0xcc,0xee,0xee,
0xee,0xcc,0xee,0xee,0xee,0xee,0xcc,0xee,0xee,0xcc,0xcc,0xcc,0xcc,0xcc,0xee,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee
};

extern int language;

// About message box /////////////////////////////////////////////////////////
void About()
{
 int i;    
 char *text1[9], *text2[6];     
 Createdialogbox(336, 231); 
 for(i = 0; i < 8; i++) text1[i] = msg[language][i + 2]; text1[i] = "";
 text1[i] = "";
 for(i = 0; i < 5; i++) text2[i] = msg[language][i + 20]; text2[i] = "";

 /*test police/////////////////////////////////////////////
 char string[8][33];
 for(i = 0; i < 8; i++)
 {
  int j;
  for(j = 0; j < 32; j++) string[i][j] = 32 + 32 * i + j;
  string[i][j] = 0;
  text2[i] = string[i];
 } 
 */

 /*version SDL/////////////////////////////////////////////
 const SDL_version *vdll = SDL_Linked_Version();
 SDL_version vsdl;
 SDL_VERSION(&vsdl);
 char versdl[50], verdll[50];
 sprintf(versdl, "Version SDL : %u.%u.%u",
         vsdl.major, vsdl.minor, vsdl.patch);
 sprintf(verdll, "Version DLL : %u.%u.%u",
         vdll->major, vdll->minor, vdll->patch);
 text2[0] = versdl;
 text2[1] = verdll;
 i = 2;
 */
 Drawmessagebox(msg[language][1], text1, text2);
}    

// Message d'erreur SDL //////////////////////////////////////////////////////
void SDL_error(int n)
{
 char string[256];    
 char *text1[3], *text2[1];     
 Createdialogbox(300, 90);    
 sprintf(string, "%i", n); 
 text1[0] = string;
 text1[1] = SDL_GetError();
 text1[2] = "";
 text2[0] = "";
 Drawmessagebox("SDL error", text1, text2);
 SDL_Delay(1000);
}

// Message d'erreur emulateur ////////////////////////////////////////////////
void Erreur(int n)
{
 char string[256];    
 char *text1[3], *text2[1];     
 Createdialogbox(300, 90);    
 sprintf(string, "%i", n); 
 text1[0] = string;
 text1[1] = msg[language][n];
 text1[2] = "";
 text2[0] = "";
 Drawmessagebox(msg[language][10], text1, text2);
}    

// Affichage de controle d'une valeur ///////////////////////////////////////
void Info(int n)
{
 char string[32];   
 char *text1[4], *text2[1];     
 sprintf(string, "0x%04x (%i)", n, n);
 Createdialogbox(120, 60);    
 text1[0] = string;
 text1[1] = "";
 text2[0] = "";
 Drawmessagebox("Info", text1, text2);
}    

// Joysticks initialization //////////////////////////////////////////////////
void Joyinit()
{ 
 //int i = 0;
 //if(joySetCapture(hwndmain, JOYSTICKID1, 0, TRUE)) i += 1; 
 //if(joySetCapture(hwndmain, JOYSTICKID2, 0, TRUE)) i += 2;
 //joySetThreshold(JOYSTICKID1, 2000); //10 trop peu pour Trust Predator
 //joySetThreshold(JOYSTICKID2, 2000); //2000 semble correct
}

// Play sound ////////////////////////////////////////////////////////////////
void Playsound(void *udata, Uint8 *stream, int bufferlength)
{
 int i;
 int mcycles; //nombre de milliemes de cycles entre deux echantillons
 int icycles; //nombre entier de cycles entre deux echantillons
 extern int sound, frequency;
 //45 cycles 6809 a 992250 Hz = un echantillon a 22050 Hz
 for(i = 0; i < bufferlength; i++)
 {
  if(pause6809) {stream[i] = 128; continue;}
  //calcul et execution du nombre de cycles entre deux echantillons
  //nbre de cycles theoriques pour la periode en cours =
  //nbre theorique + relicat arrondi precedent - cycles en trop precedent
  mcycles = frequency * 100000 / 2205;   //milliemes de cycles theoriques
  mcycles += report;                     //milliemes de cycles corriges
  icycles = mcycles / 1000;              //nbre entier de cycles a effectuer 
  report = mcycles - 1000 * icycles;     //relicat de l'arrondi a reporter
  report -= 1000 * Run(icycles);         //retrancher les milliemes en trop  
  stream[i] = sound + 128;
 }
 Testshiftkey(); //contournement bug SDL sur l'appui simultane SHIFT, ALT, CTRL
}  

//Event loop /////////////////////////////////////////////////////////////////
void Eventloop()
{
 SDL_Event event;
 extern const int xbitmap, ybitmap, ystatus;
 extern int xclient, yclient;
 extern int penbutton, xmouse, ymouse, xmove, ymove, xpen, ypen;

 while(1)
 {
  while(SDL_PollEvent(&event))
  {
   switch(event.type)
   {
    case SDL_VIDEORESIZE:
         Resizescreen(event.resize.w, event.resize.h); break;
    case SDL_MOUSEBUTTONDOWN:
         penbutton = 1; Mouseclick(); break;
    case SDL_MOUSEBUTTONUP:
         penbutton = 0; xmove = ymove = 0; break;
    case SDL_MOUSEMOTION:
         xmouse = event.motion.x; ymouse = event.motion.y;
         xpen = xmouse * xbitmap / xclient - 7;
         ypen = (ymouse - ystatus) * ybitmap / yclient - 8; 
         if(xmove) if(ymove) Dialogmove(); break;	
    case SDL_KEYDOWN:
         Keydown(event.key.keysym.sym, event.key.keysym.scancode); break;
    case SDL_KEYUP:
         Keyup(event.key.keysym.sym, event.key.keysym.scancode); break;
    case SDL_QUIT: return;
   }
  }
  SDL_Delay(20);
 }
}     

/*
// Restaurer la fenetre ///////////////////////////////////////////////////////
void Restorewindow()
{
 //Avec SDL 1.2, il semble impossible
 //- de maximiser ou restaurer la fenetre
 //- de savoir si la fenetre est maximisee ou pas
 //La fenetre initiale de SDL n'etant pas maximisee, l'idee etait d'arreter
 //puis de redemarrer le sous-systeme video pour restaurer la fenetre.
 //Il y a eu un resultat (fenetre correctement restauree) mais trop
 //d'inconvenients : perte du focus, plantages et autres phenomenes inexpliques.
 //Je crois qu'il faut attendre la version 1.3 

 extern int penbutton, xclient, yclient, ystatus; 
 extern void Resizescreen(int x, int y);
  
 //SDL_PauseAudio(1);    //Arrete l'emulation
 //SDL_Delay(200);
 SDL_QuitSubSystem(SDL_INIT_VIDEO);
 SDL_InitSubSystem(SDL_INIT_VIDEO);
 atexit(SDL_Quit);
 SDL_WM_SetIcon(SDL_LoadBMP("dcmo5.bmp"), NULL);
 SDL_WM_SetCaption(msg[language][0], NULL);            //titre fenetre
 SDL_SetCursor(SDL_CreateCursor(cursor, cursor + 48, 16, 24, 0, 0));
 Resizescreen(xclient, ystatus + yclient);
 penbutton = 0;
 //SDL_PauseAudio(0);    //Lance l'emulation
} 
*/

//Main program ///////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
 SDL_RWops *rw;
 extern char mo5rom[];
 extern int xclient, yclient, framecount;
 extern const int ystatus;

 /*
 {
  FILE *fp;
  int i, n;      
  int mnemo, type;
  char name[20];
  extern int instruction[];
  extern char mnemonique[][6];
  extern char typins[][5];
  fp = fopen("dc6809opcodes.h", "wb");
  fprintf(fp, "%s\n", "int instruction[0x300] =");
  for(i = 0; i < 0x300; i++)
  {
   mnemo = instruction[i] >> 16;    
   strcpy(name, mnemonique[mnemo]);
   strcat(name, "     ");
   name[5] = 0;
   type = (instruction[i] >> 8) & 0xff;
   n = i;
   if(i > 0x0ff) n += 0x0f00;
   fprintf(fp, "%04x: %s %s %02i  0x%06x,\n", n, name, typins[type], instruction[i] & 0xff, instruction[i]);
  } 
  fclose(fp);
 } 
 */
 
 //initialisations
 Joyinit();                   //Joysticks initialization
 Initoptions();               //Option initialization
 Initfilenames();
 Init6809registerpointers();
 Keyboardinit();              //Keyboard initialization
 Initsdlttf();                //SDL ttf initialization
 Initbuttonsurfaces();        //Initialisation des surfaces des boutons
 
 //initialize SDL video
 if(SDL_Init(SDL_INIT_VIDEO) < 0) SDL_error(2);
 atexit(SDL_Quit);
 rw = SDL_RWFromMem(dcmo5icon, sizeof(dcmo5icon));
 SDL_WM_SetIcon(SDL_LoadBMP_RW(rw, 1), NULL);  //icone fenetre
 SDL_WM_SetCaption(msg[language][0], NULL);    //titre fenetre
 SDL_SetCursor(SDL_CreateCursor(cursor, cursor + 48, 16, 24, 0, 0)); //curseur
 Initstatusbar();      //Initialisation de la surface de la barre de statut  

 //initialize SDL audio
 if(SDL_Init(SDL_INIT_AUDIO) < 0) SDL_error(3);
 audio.freq = 22050;
 audio.format = AUDIO_U8;
 audio.channels = 1;
 audio.samples = 1024; 
 audio.callback = Playsound;
 audio.userdata = NULL;
 if(SDL_OpenAudio(&audio, NULL) < 0 ) SDL_error(4);

 Hardreset();       //MO5 initialization
 framecount = 999;
 if(mo5rom[0x3fff] != 0x03) Erreur(49);
 Resizescreen(xclient, yclient + ystatus);
 SDL_PauseAudio(0); //Lance l'emulation
 
 //test events
 Eventloop();
 
 //quit
 pause6809 = 1;
 SDL_PauseAudio(1);
 Saveoptions();
 SDL_Delay(100);
 TTF_Quit();
 SDL_Quit();
 return 0;
}
